@@ -25,6 +25,9 @@
 /* Define if you have the inet_pton function.  */
 #undef HAVE_INET_PTON
 
+/* Define if you have the setproctitle function.  */
+#undef HAVE_SETPROCTITLE
+
 /* Define if you have ipv6 stack.  */
 #undef HAVE_IPV6
 
@@ -40,6 +43,9 @@
 /* whether sockaddr_in has a sin_len field */
 #undef HAVE_SIN_LEN
 
+/* whether sockaddr_un has a sun_len field */
+#undef HAVE_SUN_LEN
+
 /* whether sockaddr_in6 has a sin6_scope_id field */
 #undef HAVE_SIN6_SCOPE_ID
 
@@ -91,12 +97,18 @@
 /* Define if disable-bgp-announce option is specified. */
 #undef DISABLE_BGP_ANNOUNCE
 
-/* Define this if htnol is broken, but can be fixed with define magic */
-#undef HAVE_REPAIRABLE_HTONL
-
 /* PAM support */
 #undef USE_PAM
 
+/* TCP/IP communication between zebra and protocol daemon. */
+#undef HAVE_TCP_ZEBRA
+
+/* OSPF NSSA option */
+#undef HAVE_NSSA
+
+/* Linux netlink. */
+#undef HAVE_NETLINK
+
 /* PATHS */
 #undef PATH_ZEBRA_PID
 #undef PATH_RIPD_PID
@@ -139,8 +151,3 @@ typedef unsigned short u_int8_t;
 #ifndef HAVE_SOCKLEN_T
 typedef int socklen_t;
 #endif /* HAVE_SOCKLEN_T */
-
-/* Pseudo interfaces */
-#undef HAVE_IF_PSEUDO
-/* #define RIP_API 1 */
-#undef RIP_API
