@@ -59,7 +59,7 @@ aspath_new ()
   struct aspath *aspath;
 
   aspath = XMALLOC (MTYPE_AS_PATH, sizeof (struct aspath));
-  bzero (aspath, sizeof (struct aspath));
+  memset (aspath, 0, sizeof (struct aspath));
   return aspath;
 }
 
@@ -440,6 +440,27 @@ aspath_loop_check (struct aspath *aspath, as_t asno)
   return 0;
 }
 
+/* Command for enforce-first-as. */
+int
+aspath_firstas_check (struct aspath *aspath, as_t asno)
+{
+  caddr_t pnt;
+  struct assegment *assegment;
+
+  if (aspath == NULL)
+    return 0;
+
+  pnt = aspath->data;
+  assegment = (struct assegment *) pnt;
+
+  if (assegment
+      && assegment->type == AS_SEQUENCE
+      && assegment->asval[0] == htons (asno))
+    return 1;
+
+  return 0;
+}
+
 /* Merge as1 to as2.  as2 should be uninterned aspath. */
 struct aspath *
 aspath_merge (struct aspath *as1, struct aspath *as2)
@@ -645,9 +666,13 @@ aspath_cmp_left_confed (struct aspath *aspath1, struct aspath *aspath2)
 {
   struct assegment *seg1;
   struct assegment *seg2;
+
   as_t as1;
   as_t as2;
 
+  if (aspath1->count || aspath2->count) 
+    return 0;
+
   seg1 = (struct assegment *) aspath1->data;
   seg2 = (struct assegment *) aspath2->data;
 
@@ -856,6 +881,12 @@ aspath_empty ()
 {
   return aspath_parse (NULL, 0);
 }
+
+unsigned long
+aspath_count ()
+{
+  return ashash->count;
+}     
 
 /* 
    Theoretically, one as path can have:
@@ -1054,6 +1085,7 @@ aspath_print_all_vty (struct vty *vty)
 {
   int i;
   HashBacket *mp;
+  struct aspath *as;
 
   for (i = 0; i < HASHTABSIZE; i++)
     if ((mp = hash_head (ashash, i)) != NULL)
@@ -1061,32 +1093,8 @@ aspath_print_all_vty (struct vty *vty)
 	{
 	  vty_out (vty, "[%x:%d] (%d) ", 
 		   mp, i, ((struct aspath *)mp->data)->refcnt);
-	  aspath_print_vty (vty, mp->data);
-	  vty_out (vty, "%s", VTY_NEWLINE);
+	  as = mp->data;
+	  vty_out (vty, "%s%s", as->str, VTY_NEWLINE);
 	  mp = mp->next;
 	}
 }
-
-#ifdef ASPATH_TEST
-
-#include "regex-gnu.h"
-
-/* For test aspath functions. */
-void
-aspath_test ()
-{
-  struct aspath *as1;
-  struct aspath *as2;
-  int ret;
-
-  /* as1 = aspath_empty (); */
-  as1 = aspath_str2aspath ("");
-  as2 = aspath_str2aspath ("7675 1 2 3");
-
-  printf("%s (%d)\n", aspath_print (as1), as1->count);
-  printf("%s (%d)\n", aspath_print (as2), as2->count);
-
-  ret =  aspath_cmp_left (as1, as2);
-  printf ("result: %d\n", ret);
-}
-#endif /* ASPATH_TEST */
