@@ -112,7 +112,15 @@ bgp_timer_set (struct peer *peer)
       /* Active is waiting connection from remote peer.  And if
          connect timer is expired, change status to Connect. */
       BGP_TIMER_OFF (peer->t_start);
-      BGP_TIMER_ON (peer->t_connect, bgp_connect_timer, peer->v_connect);
+      /* If peer is passive mode, do not set connect timer. */
+      if (CHECK_FLAG (peer->flags, PEER_FLAG_PASSIVE))
+	{
+	  BGP_TIMER_OFF (peer->t_connect);
+	}
+      else
+	{
+	  BGP_TIMER_ON (peer->t_connect, bgp_connect_timer, peer->v_connect);
+	}
       BGP_TIMER_OFF (peer->t_holdtime);
       BGP_TIMER_OFF (peer->t_keepalive);
       BGP_TIMER_OFF (peer->t_asorig);
@@ -216,8 +224,7 @@ bgp_connect_timer (struct thread *thread)
   peer->t_connect = NULL;
 
   if (BGP_DEBUG (fsm, FSM))
-    zlog (peer->log, LOG_DEBUG,
-	  "%s [FSM] Timer (connect timer expire)",
+    zlog (peer->log, LOG_DEBUG, "%s [FSM] Timer (connect timer expire)",
 	  peer->host);
 
   THREAD_VAL (thread) = ConnectRetry_timer_expired;
@@ -327,12 +334,48 @@ bgp_stop (struct peer *peer)
   /* Reset all negotiated variables */
   peer->afc_nego[AFI_IP][SAFI_UNICAST] = 0;
   peer->afc_nego[AFI_IP][SAFI_MULTICAST] = 0;
+  peer->afc_nego[AFI_IP][SAFI_MPLS_VPN] = 0;
   peer->afc_nego[AFI_IP6][SAFI_UNICAST] = 0;
   peer->afc_nego[AFI_IP6][SAFI_MULTICAST] = 0;
+  peer->afc_adv[AFI_IP][SAFI_UNICAST] = 0;
+  peer->afc_adv[AFI_IP][SAFI_MULTICAST] = 0;
+  peer->afc_adv[AFI_IP][SAFI_MPLS_VPN] = 0;
+  peer->afc_adv[AFI_IP6][SAFI_UNICAST] = 0;
+  peer->afc_adv[AFI_IP6][SAFI_MULTICAST] = 0;
+  peer->afc_recv[AFI_IP][SAFI_UNICAST] = 0;
+  peer->afc_recv[AFI_IP][SAFI_MULTICAST] = 0;
+  peer->afc_recv[AFI_IP][SAFI_MPLS_VPN] = 0;
+  peer->afc_recv[AFI_IP6][SAFI_UNICAST] = 0;
+  peer->afc_recv[AFI_IP6][SAFI_MULTICAST] = 0;
 
   /* Reset route refresh flag. */
   peer->refresh_adv = 0;
-  peer->refresh_nego = 0;
+  peer->refresh_nego_old = 0;
+  peer->refresh_nego_new = 0;
+
+  /* Reset keepalive and holdtime */
+  if (peer->config & PEER_CONFIG_TIMER)
+    {
+      peer->v_keepalive = peer->keepalive;
+      peer->v_holdtime = peer->holdtime;
+    }
+  else
+    {
+      peer->v_keepalive = BGP_DEFAULT_KEEPALIVE;
+      peer->v_holdtime = BGP_DEFAULT_HOLDTIME;
+    }
+
+  /* Increment Dropped count. */
+  if (peer->status == Established)
+    {
+      peer->dropped++;
+      fsm_change_status (peer, Idle);
+#ifdef HAVE_SNMP
+      bgpTrapBackwardTransition (peer);
+#endif /* HAVE_SNMP */
+    }
+
+  peer->update_time = 0;
 
   return 0;
 }
@@ -383,6 +426,13 @@ bgp_start (struct peer *peer)
 {
   int status;
 
+  /* If the peer is passive mode, force to move to Active mode. */
+  if (CHECK_FLAG (peer->flags, PEER_FLAG_PASSIVE))
+    {
+      BGP_EVENT_ADD (peer, TCP_connection_open_failed);
+      return 0;
+    }
+
   status = bgp_connect (peer);
 
   switch (status)
@@ -494,6 +544,13 @@ bgp_establish (struct peer *peer)
   /* Clear start timer value to default. */
   peer->v_start = BGP_INIT_START_TIMER;
 
+  /* Increment established count. */
+  peer->established++;
+  fsm_change_status (peer, Established);
+#ifdef HAVE_SNMP
+  bgpTrapEstablished (peer);
+#endif /* HAVE_SNMP */
+
   /* Reset uptime, send keepalive, send current table. */
   bgp_uptime_reset (peer);
   bgp_keepalive_send (peer);
