@@ -29,7 +29,6 @@
 #include "sockunion.h"
 #include "zclient.h"
 #include "routemap.h"
-#include "newlist.h"
 
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_route.h"
@@ -50,8 +49,8 @@ bgp_if_update (struct interface *ifp)
 {
   struct bgp *bgp;
   listnode cn;
-  struct newnode *nn;
-  struct newnode *nm;
+  struct listnode *nn;
+  struct listnode *nm;
   struct peer_conf *conf;
 
   for (cn = listhead (ifp->connected); cn; nextnode (cn))
@@ -69,14 +68,14 @@ bgp_if_update (struct interface *ifp)
 	  if (IPV4_NET127 (ntohl (addr.s_addr)))
 	    continue;
 
-	  NEWLIST_LOOP (bgp_list, bgp, nn)
+	  LIST_LOOP (bgp_list, bgp, nn)
 	    {
 	      /* Respect configured router id */
 	      if (! (bgp->config & BGP_CONFIG_ROUTER_ID))
 		if (ntohl (bgp->id.s_addr) < ntohl (addr.s_addr))
 		  {
 		    bgp->id = addr;
-		    NEWLIST_LOOP (bgp->peer_conf, conf, nm)
+		    LIST_LOOP (bgp->peer_conf, conf, nm)
 		      {
 			conf->peer->local_id = addr;
 		      }
@@ -182,11 +181,11 @@ int
 bgp_interface_address_add (int command, struct zclient *zclient,
 			   zebra_size_t length)
 {
-  struct connected *c;
+  struct connected *ifc;
 
-  c = zebra_interface_address_add_read (zclient->ibuf);
+  ifc = zebra_interface_address_add_read (zclient->ibuf);
 
-  if (c == NULL)
+  if (ifc == NULL)
     return 0;
 
 #if 0
@@ -203,10 +202,10 @@ bgp_interface_address_add (int command, struct zclient *zclient,
     }
 #endif /* 0 */
 
-  bgp_if_update (c->ifp);
+  bgp_if_update (ifc->ifp);
 
-  if (if_is_up (c->ifp))
-    bgp_connected_add (c);
+  if (if_is_up (ifc->ifp))
+    bgp_connected_add (ifc);
 
   return 0;
 }
@@ -215,7 +214,19 @@ int
 bgp_interface_address_delete (int command, struct zclient *zclient,
 			      zebra_size_t length)
 {
-  ;
+  struct connected *ifc;
+
+  ifc = zebra_interface_address_delete_read (zclient->ibuf);
+
+  if (ifc == NULL)
+    return 0;
+
+  bgp_if_update (ifc->ifp);
+
+  if (if_is_up (ifc->ifp))
+    bgp_connected_delete (ifc);
+
+  connected_free (ifc);
 
   return 0;
 }
@@ -262,7 +273,7 @@ zebra_read_ipv4 (int command, struct zclient *zclient, zebra_size_t length)
     api.metric = stream_getl (s);
 
   if (command == ZEBRA_IPV4_ROUTE_ADD)
-    bgp_redistribute_add ((struct prefix *)&p, api.type);
+    bgp_redistribute_add ((struct prefix *)&p, &nexthop, api.type);
   else
     bgp_redistribute_delete ((struct prefix *)&p, api.type);
 
@@ -315,8 +326,12 @@ zebra_read_ipv6 (int command, struct zclient *zclient, zebra_size_t length)
   else
     api.metric = 0;
 
+  /* Simply ignore link-local address. */
+  if (IN6_IS_ADDR_LINKLOCAL (&p.prefix))
+    return 0;
+
   if (command == ZEBRA_IPV6_ROUTE_ADD)
-    bgp_redistribute_add ((struct prefix *) &p, api.type);
+    bgp_redistribute_add ((struct prefix *)&p, NULL, api.type);
   else
     bgp_redistribute_delete ((struct prefix *) &p, api.type);
   
@@ -1185,6 +1200,7 @@ bgp_zebra_announce (struct prefix *p, struct bgp_info *info, struct bgp *bgp)
       SET_FLAG (api.message, ZAPI_MESSAGE_NEXTHOP);
       api.nexthop_num = 1;
       api.nexthop = &nexthop;
+      api.ifindex_num = 0;
       SET_FLAG (api.message, ZAPI_MESSAGE_METRIC);
       api.metric = info->attr->med;
 
@@ -1224,8 +1240,12 @@ bgp_zebra_announce (struct prefix *p, struct bgp_info *info, struct bgp *bgp)
 	return;
 
       if (IN6_IS_ADDR_LINKLOCAL (nexthop) && ! ifindex)
-	if (info->peer->ifname)
-	  ifindex = if_nametoindex (info->peer->ifname);
+	{
+	  if (info->peer->ifname)
+	    ifindex = if_nametoindex (info->peer->ifname);
+	  else if (info->peer->nexthop.ifp)
+	    ifindex = info->peer->nexthop.ifp->ifindex;
+	}
 
       /* Make Zebra API structure. */
       api.flags = flags;
@@ -1282,6 +1302,7 @@ bgp_zebra_withdraw (struct prefix *p, struct bgp_info *info)
       SET_FLAG (api.message, ZAPI_MESSAGE_NEXTHOP);
       api.nexthop_num = 1;
       api.nexthop = &nexthop;
+      api.ifindex_num = 0;
       SET_FLAG (api.message, ZAPI_MESSAGE_METRIC);
       api.metric = info->attr->med;
 
