@@ -23,11 +23,11 @@
 
 #include "command.h"
 #include "memory.h"
-#include "newlist.h"
+#include "linklist.h"
 #include "keychain.h"
 
 /* Master list of key chain. */
-struct newlist *keychain_list;
+struct list *keychain_list;
 
 struct keychain *
 keychain_new ()
@@ -62,13 +62,13 @@ key_free (struct key *key)
 struct keychain *
 keychain_lookup (char *name)
 {
-  struct newnode *nn;
+  struct listnode *nn;
   struct keychain *keychain;
 
   if (name == NULL)
     return NULL;
 
-  NEWLIST_LOOP (keychain_list, keychain, nn)
+  LIST_LOOP (keychain_list, keychain, nn)
     {
       if (strcmp (keychain->name, name) == 0)
 	return keychain;
@@ -76,6 +76,24 @@ keychain_lookup (char *name)
   return NULL;
 }
 
+int
+key_cmp_func (struct key *k1, struct key *k2)
+{
+  if (k1->index > k2->index)
+    return 1;
+  if (k1->index < k2->index)
+    return -1;
+  return 0;
+}
+
+void
+key_delete_func (struct key *key)
+{
+  if (key->string)
+    free (key->string);
+  key_free (key);
+}
+
 struct keychain *
 keychain_get (char *name)
 {
@@ -88,38 +106,32 @@ keychain_get (char *name)
 
   keychain = keychain_new ();
   keychain->name = strdup (name);
-  keychain->key = newlist_new ();
-  newnode_add (keychain_list, keychain);
+  keychain->key = list_new ();
+  keychain->key->cmp = (int (*)(void *, void *)) key_cmp_func;
+  keychain->key->del = (void (*)(void *)) key_delete_func;
+  listnode_add (keychain_list, keychain);
 
   return keychain;
 }
 
-void
-key_delete_func (struct key *key)
-{
-  if (key->string)
-    free (key->string);
-  key_free (key);
-}
-
 void
 keychain_delete (struct keychain *keychain)
 {
   if (keychain->name)
     free (keychain->name);
 
-  newlist_delete (keychain->key);
-  newnode_delete (keychain_list, keychain);
+  list_delete (keychain->key);
+  listnode_delete (keychain_list, keychain);
   keychain_free (keychain);
 }
 
 struct key *
 key_lookup (struct keychain *keychain, u_int32_t index)
 {
-  struct newnode *nn;
+  struct listnode *nn;
   struct key *key;
 
-  NEWLIST_LOOP (keychain->key, key, nn)
+  LIST_LOOP (keychain->key, key, nn)
     {
       if (key->index == index)
 	return key;
@@ -130,13 +142,13 @@ key_lookup (struct keychain *keychain, u_int32_t index)
 struct key *
 key_lookup_for_accept (struct keychain *keychain, u_int32_t index)
 {
-  struct newnode *nn;
+  struct listnode *nn;
   struct key *key;
   time_t now;
 
   now = time (NULL);
 
-  NEWLIST_LOOP (keychain->key, key, nn)
+  LIST_LOOP (keychain->key, key, nn)
     {
       if (key->index == index)
 	{
@@ -154,13 +166,13 @@ key_lookup_for_accept (struct keychain *keychain, u_int32_t index)
 struct key *
 key_match_for_accept (struct keychain *keychain, char *auth_str)
 {
-  struct newnode *nn;
+  struct listnode *nn;
   struct key *key;
   time_t now;
 
   now = time (NULL);
 
-  NEWLIST_LOOP (keychain->key, key, nn)
+  LIST_LOOP (keychain->key, key, nn)
     {
       if (key->accept.start == 0 ||
 	  (key->accept.start <= now &&
@@ -174,13 +186,13 @@ key_match_for_accept (struct keychain *keychain, char *auth_str)
 struct key *
 key_lookup_for_send (struct keychain *keychain)
 {
-  struct newnode *nn;
+  struct listnode *nn;
   struct key *key;
   time_t now;
 
   now = time (NULL);
 
-  NEWLIST_LOOP (keychain->key, key, nn)
+  LIST_LOOP (keychain->key, key, nn)
     {
       if (key->send.start == 0)
 	return key;
@@ -204,7 +216,7 @@ key_get (struct keychain *keychain, u_int32_t index)
 
   key = key_new ();
   key->index = index;
-  newnode_add (keychain->key, key);
+  listnode_add_sort (keychain->key, key);
 
   return key;
 }
@@ -212,7 +224,7 @@ key_get (struct keychain *keychain, u_int32_t index)
 void
 key_delete (struct keychain *keychain, struct key *key)
 {
-  newnode_delete (keychain->key, key);
+  listnode_delete (keychain->key, key);
 
   if (key->string)
     free (key->string);
@@ -858,14 +870,14 @@ struct cmd_node keychain_node =
 {
   KEYCHAIN_NODE,
   "%s(config-keychain)# ",
-  0
+  1
 };
 
 struct cmd_node keychain_key_node =
 {
   KEYCHAIN_KEY_NODE,
   "%s(config-keychain-key)# ",
-  0
+  1
 };
 
 int
@@ -886,15 +898,15 @@ keychain_config_write (struct vty *vty)
 {
   struct keychain *keychain;
   struct key *key;
-  struct newnode *nn;
-  struct newnode *nm;
+  struct listnode *nn;
+  struct listnode *nm;
   char buf[BUFSIZ];
 
-  NEWLIST_LOOP (keychain_list, keychain, nn)
+  LIST_LOOP (keychain_list, keychain, nn)
     {
       vty_out (vty, "key chain %s%s", keychain->name, VTY_NEWLINE);
       
-      NEWLIST_LOOP (keychain->key, key, nm)
+      LIST_LOOP (keychain->key, key, nm)
 	{
 	  vty_out (vty, " key %d%s", key->index, VTY_NEWLINE);
 
@@ -945,8 +957,7 @@ keychain_config_write (struct vty *vty)
 void
 keychain_init ()
 {
-  keychain_list = newlist_new ();
-  keychain_list->del = (void (*)(void *)) key_delete_func;
+  keychain_list = list_new ();
 
   install_node (&keychain_node, keychain_config_write);
   install_node (&keychain_key_node, NULL);
@@ -958,9 +969,16 @@ keychain_init ()
   install_element (CONFIG_NODE, &no_key_chain_cmd);
   install_element (KEYCHAIN_NODE, &key_cmd);
   install_element (KEYCHAIN_NODE, &no_key_cmd);
+
+  install_element (KEYCHAIN_NODE, &key_chain_cmd);
+  install_element (KEYCHAIN_NODE, &no_key_chain_cmd);
+
   install_element (KEYCHAIN_KEY_NODE, &key_string_cmd);
   install_element (KEYCHAIN_KEY_NODE, &no_key_string_cmd);
 
+  install_element (KEYCHAIN_KEY_NODE, &key_chain_cmd);
+  install_element (KEYCHAIN_KEY_NODE, &no_key_chain_cmd);
+
   install_element (KEYCHAIN_KEY_NODE, &key_cmd);
   install_element (KEYCHAIN_KEY_NODE, &no_key_cmd);
 
