@@ -72,6 +72,9 @@ char *vty_cwd = NULL;
 
 /* Configure lock. */
 static int vty_config;
+
+/* Login password check. */
+static int no_password_check = 0;
 
 /* VTY standard output function. */
 int
@@ -127,7 +130,7 @@ vvty_out (struct vty *vty, const char *format, va_list va)
 
 /* Output current time to the vty. */
 void
-vty_time_print (struct vty *vty)
+vty_time_print (struct vty *vty, int cr)
 {
   time_t clock;
   struct tm *tm;
@@ -144,7 +147,10 @@ vty_time_print (struct vty *vty)
       zlog (NULL, LOG_INFO, "strftime error");
       return;
     }
-  vty_out (vty, "%s\n", buf);
+  if (cr)
+    vty_out (vty, "%s\n", buf);
+  else
+    vty_out (vty, "%s ", buf);
 
   return;
 }
@@ -225,7 +231,7 @@ vty_new ()
   struct vty *new = XMALLOC (MTYPE_VTY, sizeof (struct vty));
   bzero (new, sizeof (struct vty));
 
-  new->obuf = (struct buffer *) buffer_new (BUFFER_VTY, 100);
+  new->obuf = (struct buffer *) buffer_new (100);
   new->buf = XMALLOC (MTYPE_VTY, VTY_BUFSIZ);
   new->max = VTY_BUFSIZ;
   new->sb_buffer = NULL;
@@ -574,9 +580,6 @@ vty_end_config (struct vty *vty)
       /* Nothing to do. */
       break;
     case CONFIG_NODE:
-      vty_config_unlock (vty);
-      vty->node = ENABLE_NODE;
-      break;
     case INTERFACE_NODE:
     case ZEBRA_NODE:
     case RIP_NODE:
@@ -590,6 +593,7 @@ vty_end_config (struct vty *vty)
     case KEYCHAIN_KEY_NODE:
     case MASC_NODE:
     case VTY_NODE:
+      vty_config_unlock (vty);
       vty->node = ENABLE_NODE;
       break;
     default:
@@ -981,9 +985,6 @@ vty_stop_input (struct vty *vty)
       /* Nothing to do. */
       break;
     case CONFIG_NODE:
-      vty_config_unlock (vty);
-      vty->node = ENABLE_NODE;
-      break;
     case INTERFACE_NODE:
     case ZEBRA_NODE:
     case RIP_NODE:
@@ -996,6 +997,7 @@ vty_stop_input (struct vty *vty)
     case KEYCHAIN_KEY_NODE:
     case MASC_NODE:
     case VTY_NODE:
+      vty_config_unlock (vty);
       vty->node = ENABLE_NODE;
       break;
     default:
@@ -1484,10 +1486,12 @@ vty_flush (struct thread *thread)
 	  if (vty->status == VTY_CLOSE)
 	    vty_close (vty);
 	  else
-	    vty->status = VTY_NORMAL;
+	    {
+	      vty->status = VTY_NORMAL;
 	  
-	  if (vty->lines == 0)
-	    vty_event (VTY_READ, vty_sock, vty);
+	      if (vty->lines == 0)
+		vty_event (VTY_READ, vty_sock, vty);
+	    }
 	}
       else
 	{
@@ -1512,7 +1516,15 @@ vty_create (int vty_sock, union sockunion *su)
   vty->fd = vty_sock;
   vty->type = VTY_TERM;
   vty->address = sockunion_su2str (su);
-  vty->node = AUTH_NODE;
+  if (no_password_check)
+    {
+      if (host.advanced)
+	vty->node = ENABLE_NODE;
+      else
+	vty->node = VIEW_NODE;
+    }
+  else
+    vty->node = AUTH_NODE;
   vty->fail = 0;
   vty->cp = 0;
   vty_clear_buf (vty);
@@ -1529,20 +1541,24 @@ vty_create (int vty_sock, union sockunion *su)
     vty->lines = -1;
   vty->iac = 0;
   vty->iac_sb_in_progress = 0;
-  vty->sb_buffer = buffer_new(BUFFER_STRING, 1024);
+  vty->sb_buffer = buffer_new (1024);
 
-  /* Vty is not available if password isn't set. */
-  if (host.password == NULL && host.password_encrypt == NULL)
+  if (! no_password_check)
     {
-      vty_out (vty, "Vty password is not set.%s", VTY_NEWLINE);
-      vty->status = VTY_CLOSE;
-      vty_close (vty);
-      return NULL;
+      /* Vty is not available if password isn't set. */
+      if (host.password == NULL && host.password_encrypt == NULL)
+	{
+	  vty_out (vty, "Vty password is not set.%s", VTY_NEWLINE);
+	  vty->status = VTY_CLOSE;
+	  vty_close (vty);
+	  return NULL;
+	}
     }
 
   /* Say hello to the world. */
   vty_hello (vty);
-  vty_out (vty, "%sUser Access Verification%s%s", VTY_NEWLINE, VTY_NEWLINE, VTY_NEWLINE);
+  if (! no_password_check)
+    vty_out (vty, "%sUser Access Verification%s%s", VTY_NEWLINE, VTY_NEWLINE, VTY_NEWLINE);
 
   /* Setting up terminal. */
   vty_will_echo (vty);
@@ -1665,12 +1681,13 @@ vty_serv_sock_addrinfo (unsigned short port)
   req.ai_family = AF_UNSPEC;
   req.ai_socktype = SOCK_STREAM;
   sprintf (port_str, "%d", port);
+  port_str[sizeof (port_str) - 1] = '\0';
 
   ret = getaddrinfo (NULL, port_str, &req, &ainfo);
 
   if (ret != 0)
     {
-      fprintf (stderr, "getaddrinfo failed: %s\n", strerror (errno));
+      fprintf (stderr, "getaddrinfo failed: %s\n", gai_strerror (ret));
       exit (1);
     }
 
@@ -1678,6 +1695,13 @@ vty_serv_sock_addrinfo (unsigned short port)
 
   do
     {
+      if (ainfo->ai_family != AF_INET
+#ifdef HAVE_IPV6
+	  && ainfo->ai_family != AF_INET6
+#endif /* HAVE_IPV6 */
+	  )
+	continue;
+
       sock = socket (ainfo->ai_family, ainfo->ai_socktype, ainfo->ai_protocol);
       if (sock < 0)
 	continue;
@@ -1687,11 +1711,17 @@ vty_serv_sock_addrinfo (unsigned short port)
 
       ret = bind (sock, ainfo->ai_addr, ainfo->ai_addrlen);
       if (ret < 0)
+	{
+	  close (sock);	/* Avoid sd leak. */
 	continue;
+	}
 
       ret = listen (sock, 3);
       if (ret < 0) 
+	{
+	  close (sock);	/* Avoid sd leak. */
 	continue;
+	}
 
       vty_event (VTY_SERV, sock, NULL);
     }
@@ -1714,19 +1744,27 @@ vty_serv_sock_family (unsigned short port, int family)
 
   /* Make new socket. */
   accept_sock = sockunion_stream_socket (&su);
+  if (accept_sock < 0)
+    return;
 
   /* This is server, so reuse address. */
   sockopt_reuseaddr (accept_sock);
   sockopt_reuseport (accept_sock);
 
   /* Bind socket to universal address and given port. */
-  sockunion_bind (accept_sock, &su, port, NULL);
+  ret = sockunion_bind (accept_sock, &su, port, NULL);
+  if (ret < 0)
+    {
+      close (accept_sock);	/* Avoid sd leak. */
+      return;
+    }
 
   /* Listen socket under queue 3. */
   ret = listen (accept_sock, 3);
   if (ret < 0) 
     {
       zlog (NULL, LOG_WARNING, "can't listen socket");
+      close (accept_sock);	/* Avoid sd leak. */
       return;
     }
 
@@ -1743,7 +1781,7 @@ void
 vty_serv_un (char *path)
 {
   int ret;
-  int sock;
+  int sock, len;
   struct sockaddr_un serv;
   mode_t old_mask;
 
@@ -1765,16 +1803,27 @@ vty_serv_un (char *path)
   memset (&serv, 0, sizeof (struct sockaddr_un));
   serv.sun_family = AF_UNIX;
   strncpy (serv.sun_path, path, strlen (path));
+#ifdef HAVE_SUN_LEN
+  len = serv.sun_len = SUN_LEN(&serv);
+#else
+  len = sizeof (serv.sun_family) + strlen (serv.sun_path);
+#endif /* HAVE_SUN_LEN */
 
-  ret = bind (sock, (struct sockaddr *) &serv, sizeof (struct sockaddr_un));
+  ret = bind (sock, (struct sockaddr *) &serv, len);
   if (ret < 0)
     {
       perror ("bind");
-      close (sock);
+      close (sock);	/* Avoid sd leak. */
       return;
     }
 
-  listen (sock, 5);
+  ret = listen (sock, 5);
+  if (ret < 0)
+    {
+      perror ("listen");
+      close (sock);	/* Avoid sd leak. */
+      return;
+    }
 
   umask (old_mask);
 
@@ -1838,7 +1887,6 @@ vtysh_read (struct thread *thread)
   nbytes = read (sock, buf, VTY_READ_BUFSIZ);
   if (nbytes <= 0)
     {
-
       vty_close (vty);
 #ifdef VTYSH_DEBUG
       printf ("close vtysh\n");
@@ -1850,14 +1898,18 @@ vtysh_read (struct thread *thread)
   printf ("line: %s\n", buf);
 #endif /* VTYSH_DEBUG */
 
+  vty_ensure (vty, nbytes);
+  memcpy (vty->buf, buf, nbytes);
+  
   /* Pass this line to parser. */
-  vty->buf = buf;
   ret = vty_execute (vty);
-  vty->buf = NULL;
+
+  vty_clear_buf (vty);
 
   /* Return result. */
 #ifdef VTYSH_DEBUG
   printf ("result: %d\n", ret);
+  printf ("vtysh node: %d\n", vty->node);
 #endif /* VTYSH_DEBUG */
 
   header[3] = ret;
@@ -1975,8 +2027,6 @@ vty_read_file (FILE *confp)
   /* Execute configuration file */
   ret = config_from_file (vty, confp);
 
-  vty_close (vty);
-
   if (ret != CMD_SUCCESS) 
     {
       switch (ret)
@@ -1990,8 +2040,67 @@ vty_read_file (FILE *confp)
 	}
       fprintf (stderr, "Error occured during reading below line.\n%s\n", 
 	       vty->buf);
+      vty_close (vty);
       exit (1);
     }
+
+  vty_close (vty);
+}
+
+FILE *
+vty_use_backup_config (char *fullpath)
+{
+  char *fullpath_sav, *fullpath_tmp;
+  FILE *ret = NULL;
+  struct stat buf;
+  int tmp, sav;
+  int c;
+  char buffer[512];
+  
+  fullpath_sav = malloc (strlen (fullpath) + strlen (CONF_BACKUP_EXT) + 1);
+  strcpy (fullpath_sav, fullpath);
+  strcat (fullpath_sav, CONF_BACKUP_EXT);
+  if (stat (fullpath_sav, &buf) == -1)
+    {
+      free (fullpath_sav);
+      return NULL;
+    }
+
+  fullpath_tmp = malloc (strlen (fullpath) + 8);
+  sprintf (fullpath_tmp, "%s.XXXXXX", fullpath);
+  
+  /* Open file to configuration write. */
+  tmp = mkstemp (fullpath_tmp);
+  if (tmp < 0)
+    {
+      free (fullpath_sav);
+      free (fullpath_tmp);
+      return NULL;
+    }
+
+  sav = open (fullpath_sav, O_RDONLY);
+  if (sav < 0)
+    {
+      free (fullpath_sav);
+      free (fullpath_tmp);
+      unlink (fullpath_tmp);
+      return NULL;
+    }
+  
+  while((c = read (sav, buffer, 512)) > 0)
+    write (tmp, buffer, c);
+  
+  close (sav);
+  close (tmp);
+  
+  if (link (fullpath_tmp, fullpath) == 0)
+    ret = fopen (fullpath, "r");
+
+  unlink (fullpath_tmp);
+  
+  free (fullpath_sav);
+  free (fullpath_tmp);
+  return fopen (fullpath, "r");
 }
 
 /* Read up configuration file from file_name. */
@@ -2021,16 +2130,30 @@ vty_read_config (char *config_file,
 
       if (confp == NULL)
 	{
-	  fprintf (stderr, "can't open configuration file [%s]\n", 
-		   config_file);
-	  exit(1);
+	  confp = vty_use_backup_config (fullpath);
+	  if (confp)
+	    fprintf (stderr, "WARNING: using backup configuration file!\n");
+	  else
+	    {
+	      fprintf (stderr, "can't open configuration file [%s]\n", 
+		       config_file);
+	      exit(1);
+	    }
 	}
     }
   else
     {
       /* Relative path configuration file open. */
       if (config_current_dir)
-	confp = fopen (config_current_dir, "r");
+	{
+	  confp = fopen (config_current_dir, "r");
+	  if (confp == NULL)
+	    {
+	      confp = vty_use_backup_config (config_current_dir);
+	      if (confp)
+		fprintf (stderr, "WARNING: using backup configuration file!\n");
+	    }
+	}
 
       /* If there is no relative path exists, open system default file. */
       if (confp == NULL)
@@ -2038,9 +2161,18 @@ vty_read_config (char *config_file,
 	  confp = fopen (config_default_dir, "r");
 	  if (confp == NULL)
 	    {
-	      fprintf (stderr, "can't open configuration file [%s]\n",
-		       config_default_dir);
-	      exit (1);
+	      confp = vty_use_backup_config (config_default_dir);
+	      if (confp)
+		{
+		  fprintf (stderr, "WARNING: using backup configuration file!\n");
+		  fullpath = config_default_dir;
+		}
+	      else
+		{
+		  fprintf (stderr, "can't open configuration file [%s]\n",
+			   config_default_dir);
+		  exit (1);
+		}
 	    }      
 	  else
 	    fullpath = config_default_dir;
@@ -2072,6 +2204,7 @@ vty_log (const char *proto_str, const char *format, va_list va)
     if ((vty = vector_slot (vtyvec, i)) != NULL)
       if (vty->monitor)
 	{
+	  vty_time_print (vty, 0);
 	  vty_out (vty, "%s: ", proto_str);
 	  vvty_out (vty, format, va);
 	  vty_out (vty, "\r\n");
@@ -2175,8 +2308,8 @@ DEFUN (config_who,
 DEFUN (line_vty,
        line_vty_cmd,
        "line vty",
-       "Configure vty\n"
-       "Configure vty\n")
+       "Configure a terminal line\n"
+       "Virtual terminal\n")
 {
   vty->node = VTY_NODE;
   return CMD_SUCCESS;
@@ -2218,9 +2351,9 @@ DEFUN (exec_timeout_min,
 DEFUN (exec_timeout_sec,
        exec_timeout_sec_cmd,
        "exec-timeout <0-35791> <0-2147483>",
-       "Set timeout value\n"
-       "Timeout value in minutes\n"
-       "Timeout value in seconds\n")
+       "Set the EXEC timeout\n"
+       "Timeout in minutes\n"
+       "Timeout in seconds\n")
 {
   return exec_timeout (vty, argv[0], argv[1]);
 }
@@ -2229,7 +2362,7 @@ DEFUN (no_exec_timeout,
        no_exec_timeout_cmd,
        "no exec-timeout",
        NO_STR
-       "Unset timeout\n")
+       "Set the EXEC timeout\n")
 {
   return exec_timeout (vty, NULL, NULL);
 }
@@ -2237,9 +2370,9 @@ DEFUN (no_exec_timeout,
 /* Set vty access class. */
 DEFUN (vty_access_class,
        vty_access_class_cmd,
-       "access-class ACCESS-LIST",
-       "Apply access list to vty\n"
-       "Access list name\n")
+       "access-class WORD",
+       "Filter connections based on an IP access list\n"
+       "IP access list\n")
 {
   if (vty_accesslist_name)
     XFREE(MTYPE_VTY, vty_accesslist_name);
@@ -2252,10 +2385,10 @@ DEFUN (vty_access_class,
 /* Clear vty access class. */
 DEFUN (no_vty_access_class,
        no_vty_access_class_cmd,
-       "no access-class [ACCESS-LIST]",
+       "no access-class [WORD]",
        NO_STR
-       "Access list to remove from vty\n"
-       "Access list name\n")
+       "Filter connections based on an IP access list\n"
+       "IP access list\n")
 {
   if (! vty_accesslist_name || (argc && strcmp(vty_accesslist_name, argv[0])))
     {
@@ -2275,10 +2408,10 @@ DEFUN (no_vty_access_class,
 /* Set vty access class. */
 DEFUN (vty_ipv6_access_class,
        vty_ipv6_access_class_cmd,
-       "ipv6 access-class ACCESS-LIST",
+       "ipv6 access-class WORD",
        IPV6_STR
-       "Apply access list to vty\n"
-       "Access list name\n")
+       "Filter connections based on an IP access list\n"
+       "IPv6 access list\n")
 {
   if (vty_ipv6_accesslist_name)
     XFREE(MTYPE_VTY, vty_ipv6_accesslist_name);
@@ -2291,11 +2424,11 @@ DEFUN (vty_ipv6_access_class,
 /* Clear vty access class. */
 DEFUN (no_vty_ipv6_access_class,
        no_vty_ipv6_access_class_cmd,
-       "no ipv6 access-class [ACCESS-LIST]",
+       "no ipv6 access-class [WORD]",
        NO_STR
        IPV6_STR
-       "Access list to remove from vty\n"
-       "Access list name\n")
+       "Filter connections based on an IP access list\n"
+       "IPv6 access list\n")
 {
   if (! vty_ipv6_accesslist_name ||
       (argc && strcmp(vty_ipv6_accesslist_name, argv[0])))
@@ -2313,6 +2446,26 @@ DEFUN (no_vty_ipv6_access_class,
 }
 #endif /* HAVE_IPV6 */
 
+/* vty login. */
+DEFUN (vty_login,
+       vty_login_cmd,
+       "login",
+       "Enable password checking\n")
+{
+  no_password_check = 0;
+  return CMD_SUCCESS;
+}
+
+DEFUN (no_vty_login,
+       no_vty_login_cmd,
+       "no login",
+       NO_STR
+       "Enable password checking\n")
+{
+  no_password_check = 1;
+  return CMD_SUCCESS;
+}
+
 DEFUN (service_advanced_vty,
        service_advanced_vty_cmd,
        "service advanced-vty",
@@ -2384,37 +2537,35 @@ DEFUN (show_history,
 int
 vty_config_write (struct vty *vty)
 {
-  int write = 0;
+  vty_out (vty, "line vty%s", VTY_NEWLINE);
 
-  if ((vty_timeout_val != VTY_TIMEOUT_DEFAULT) || 
-      vty_accesslist_name ||
-      vty_ipv6_accesslist_name)
-    {
-      vty_out (vty, "line vty%s", VTY_NEWLINE);
+  if (vty_accesslist_name)
+    vty_out (vty, " access-class %s%s",
+	     vty_accesslist_name, VTY_NEWLINE);
 
-      /* exec-timeout */
-      if (vty_timeout_val != VTY_TIMEOUT_DEFAULT)
-	vty_out (vty, " exec-timeout %d %d%s", 
-		 vty_timeout_val / 60,
-		 vty_timeout_val % 60, VTY_NEWLINE);
+  if (vty_ipv6_accesslist_name)
+    vty_out (vty, " ipv6 access-class %s%s",
+	     vty_ipv6_accesslist_name, VTY_NEWLINE);
 
-      if (vty_accesslist_name)
-	vty_out (vty, " access-class %s%s",
-		 vty_accesslist_name, VTY_NEWLINE);
+  /* exec-timeout */
+  if (vty_timeout_val != VTY_TIMEOUT_DEFAULT)
+    vty_out (vty, " exec-timeout %d %d%s", 
+	     vty_timeout_val / 60,
+	     vty_timeout_val % 60, VTY_NEWLINE);
 
-      if (vty_ipv6_accesslist_name)
-	vty_out (vty, " ipv6 access-class %s%s",
-		 vty_ipv6_accesslist_name, VTY_NEWLINE);
+  /* login */
+  if (no_password_check)
+    vty_out (vty, " no login%s", VTY_NEWLINE);
 
-      write++;
-    }
-  return write;
+  vty_out (vty, "!%s", VTY_NEWLINE);
+
+  return CMD_SUCCESS;
 }
 
 struct cmd_node vty_node =
 {
   VTY_NODE,
-  "%s(config-vty)# ",
+  "%s(config-line)# ",
 };
 
 /* Reset all VTY status. */
@@ -2438,6 +2589,47 @@ vty_reset ()
       {
 	thread_cancel (vty_serv_thread);
 	vector_slot (Vvty_serv_thread, i) = NULL;
+        close (i);
+      }
+
+  vty_timeout_val = VTY_TIMEOUT_DEFAULT;
+
+  if (vty_accesslist_name)
+    {
+      XFREE(MTYPE_VTY, vty_accesslist_name);
+      vty_accesslist_name = NULL;
+    }
+
+  if (vty_ipv6_accesslist_name)
+    {
+      XFREE(MTYPE_VTY, vty_ipv6_accesslist_name);
+      vty_ipv6_accesslist_name = NULL;
+    }
+}
+
+/* for ospf6d easy temprary reload function */
+/* vty_reset + close accept socket */
+void
+vty_finish ()
+{
+  int i;
+  struct vty *vty;
+  struct thread *vty_serv_thread;
+
+  for (i = 0; i < vector_max (vtyvec); i++)
+    if ((vty = vector_slot (vtyvec, i)) != NULL)
+      {
+	buffer_reset (vty->obuf);
+	vty->status = VTY_CLOSE;
+	vty_close (vty);
+      }
+
+  for (i = 0; i < vector_max (Vvty_serv_thread); i++)
+    if ((vty_serv_thread = vector_slot (Vvty_serv_thread, i)) != NULL)
+      {
+	thread_cancel (vty_serv_thread);
+	vector_slot (Vvty_serv_thread, i) = NULL;
+        close (i);
       }
 
   vty_timeout_val = VTY_TIMEOUT_DEFAULT;
@@ -2522,6 +2714,8 @@ vty_init ()
   install_element (VTY_NODE, &no_exec_timeout_cmd);
   install_element (VTY_NODE, &vty_access_class_cmd);
   install_element (VTY_NODE, &no_vty_access_class_cmd);
+  install_element (VTY_NODE, &vty_login_cmd);
+  install_element (VTY_NODE, &no_vty_login_cmd);
 #ifdef HAVE_IPV6
   install_element (VTY_NODE, &vty_ipv6_access_class_cmd);
   install_element (VTY_NODE, &no_vty_ipv6_access_class_cmd);
