@@ -29,7 +29,9 @@
 #include "thread.h"
 #include "zclient.h"
 #include "memory.h"
+#include "table.h"
 
+#include "zebra/rib.h"
 #include "zebra/zserv.h"
 
 /* Zebra client events. */
@@ -161,6 +163,39 @@ zclient_socket ()
   return sock;
 }
 
+/* For sockaddr_un. */
+#include <sys/un.h>
+
+int
+zclient_socket_un (char *path)
+{
+  int ret;
+  int sock, len;
+  struct sockaddr_un addr;
+
+  sock = socket (AF_UNIX, SOCK_STREAM, 0);
+  if (sock < 0)
+    return -1;
+  
+  /* Make server socket. */ 
+  memset (&addr, 0, sizeof (struct sockaddr_un));
+  addr.sun_family = AF_UNIX;
+  strncpy (addr.sun_path, path, strlen (path));
+#ifdef HAVE_SUN_LEN
+  len = addr.sun_len = SUN_LEN(&addr);
+#else
+  len = sizeof (addr.sun_family) + strlen (addr.sun_path);
+#endif /* HAVE_SUN_LEN */
+
+  ret = connect (sock, (struct sockaddr *) &addr, len);
+  if (ret < 0)
+    {
+      close (sock);
+      return -1;
+    }
+  return sock;
+}
+
 /* Send simple Zebra message. */
 int
 zebra_message_send (struct zclient *zclient, int command)
@@ -200,7 +235,11 @@ zclient_start (struct zclient *zclient)
     return 0;
 
   /* Make socket. */
+#ifdef HAVE_TCP_ZEBRA
   zclient->sock = zclient_socket ();
+#else
+  zclient->sock = zclient_socket_un (ZEBRA_SERV_PATH);
+#endif /* HAVE_TCP_ZEBRA */
   if (zclient->sock < 0)
     {
       if (zclient_debug)
@@ -278,16 +317,20 @@ zapi_ipv4_add (struct zclient *zclient, struct prefix_ipv4 *p,
   /* Nexthop, ifindex, distance and metric information. */
   if (CHECK_FLAG (api->message, ZAPI_MESSAGE_NEXTHOP))
     {
-      stream_putc (s, api->nexthop_num);
+      stream_putc (s, api->nexthop_num + api->ifindex_num);
+
       for (i = 0; i < api->nexthop_num; i++)
-	stream_put_in_addr (s, api->nexthop[i]);
-    }
-  if (CHECK_FLAG (api->message, ZAPI_MESSAGE_IFINDEX))
-    {
-      stream_putc (s, api->ifindex_num);
+	{
+	  stream_putc (s, ZEBRA_NEXTHOP_IPV4);
+	  stream_put_in_addr (s, api->nexthop[i]);
+	}
       for (i = 0; i < api->ifindex_num; i++)
-	stream_putl (s, api->ifindex[i]);
+	{
+	  stream_putc (s, ZEBRA_NEXTHOP_IFINDEX);
+	  stream_putl (s, api->ifindex[i]);
+	}
     }
+
   if (CHECK_FLAG (api->message, ZAPI_MESSAGE_DISTANCE))
     stream_putc (s, api->distance);
   if (CHECK_FLAG (api->message, ZAPI_MESSAGE_METRIC))
@@ -328,16 +371,20 @@ zapi_ipv4_delete (struct zclient *zclient, struct prefix_ipv4 *p,
   /* Nexthop, ifindex, distance and metric information. */
   if (CHECK_FLAG (api->message, ZAPI_MESSAGE_NEXTHOP))
     {
-      stream_putc (s, api->nexthop_num);
+      stream_putc (s, api->nexthop_num + api->ifindex_num);
+
       for (i = 0; i < api->nexthop_num; i++)
-	stream_put_in_addr (s, api->nexthop[i]);
-    }
-  if (CHECK_FLAG (api->message, ZAPI_MESSAGE_IFINDEX))
-    {
-      stream_putc (s, api->ifindex_num);
+	{
+	  stream_putc (s, ZEBRA_NEXTHOP_IPV4);
+	  stream_put_in_addr (s, api->nexthop[i]);
+	}
       for (i = 0; i < api->ifindex_num; i++)
-	stream_putl (s, api->ifindex[i]);
+	{
+	  stream_putc (s, ZEBRA_NEXTHOP_IFINDEX);
+	  stream_putl (s, api->ifindex[i]);
+	}
     }
+
   if (CHECK_FLAG (api->message, ZAPI_MESSAGE_DISTANCE))
     stream_putc (s, api->distance);
   if (CHECK_FLAG (api->message, ZAPI_MESSAGE_METRIC))
@@ -379,16 +426,20 @@ zapi_ipv6_add (struct zclient *zclient, struct prefix_ipv6 *p,
   /* Nexthop, ifindex, distance and metric information. */
   if (CHECK_FLAG (api->message, ZAPI_MESSAGE_NEXTHOP))
     {
-      stream_putc (s, api->nexthop_num);
+      stream_putc (s, api->nexthop_num + api->ifindex_num);
+
       for (i = 0; i < api->nexthop_num; i++)
-	stream_write (s, (u_char *)api->nexthop[i], 16);
-    }
-  if (CHECK_FLAG (api->message, ZAPI_MESSAGE_IFINDEX))
-    {
-      stream_putc (s, api->ifindex_num);
+	{
+	  stream_putc (s, ZEBRA_NEXTHOP_IPV6);
+	  stream_write (s, (u_char *)api->nexthop[i], 16);
+	}
       for (i = 0; i < api->ifindex_num; i++)
-	stream_putl (s, api->ifindex[i]);
+	{
+	  stream_putc (s, ZEBRA_NEXTHOP_IFINDEX);
+	  stream_putl (s, api->ifindex[i]);
+	}
     }
+
   if (CHECK_FLAG (api->message, ZAPI_MESSAGE_DISTANCE))
     stream_putc (s, api->distance);
   if (CHECK_FLAG (api->message, ZAPI_MESSAGE_METRIC))
@@ -429,16 +480,20 @@ zapi_ipv6_delete (struct zclient *zclient, struct prefix_ipv6 *p,
   /* Nexthop, ifindex, distance and metric information. */
   if (CHECK_FLAG (api->message, ZAPI_MESSAGE_NEXTHOP))
     {
-      stream_putc (s, api->nexthop_num);
+      stream_putc (s, api->nexthop_num + api->ifindex_num);
+
       for (i = 0; i < api->nexthop_num; i++)
-	stream_write (s, (u_char *)api->nexthop[i], 16);
-    }
-  if (CHECK_FLAG (api->message, ZAPI_MESSAGE_IFINDEX))
-    {
-      stream_putc (s, api->ifindex_num);
+	{
+	  stream_putc (s, ZEBRA_NEXTHOP_IPV6);
+	  stream_write (s, (u_char *)api->nexthop[i], 16);
+	}
       for (i = 0; i < api->ifindex_num; i++)
-	stream_putl (s, api->ifindex[i]);
+	{
+	  stream_putc (s, ZEBRA_NEXTHOP_IFINDEX);
+	  stream_putl (s, api->ifindex[i]);
+	}
     }
+
   if (CHECK_FLAG (api->message, ZAPI_MESSAGE_DISTANCE))
     stream_putc (s, api->distance);
   if (CHECK_FLAG (api->message, ZAPI_MESSAGE_METRIC))
@@ -501,15 +556,10 @@ zebra_interface_add_read (struct stream *s)
   ifp->metric = stream_getl (s);
   ifp->mtu = stream_getl (s);
   ifp->bandwidth = stream_getl (s);
-
-#ifdef HAVE_IF_PSEUDO
-  if (IS_IF_PSEUDO(ifp)){
-    IF_PSEUDO_SET(ifp);
-  }
-  else{
-    IF_PSEUDO_UNSET(ifp);
-  }
-#endif /* HAVE_IF_PSEUDO */
+#ifndef HAVE_SOCKADDR_DL
+  ifp->hw_addr_len = stream_getl (s);
+  stream_get (ifp->hw_addr, s, ifp->hw_addr_len);
+#endif /* HAVE_SOCKADDR_DL */
   
   return ifp;
 }
@@ -540,15 +590,6 @@ zebra_interface_state_read (struct stream *s)
   ifp->mtu = stream_getl (s);
   ifp->bandwidth = stream_getl (s);
 
-#ifdef HAVE_IF_PSEUDO  
-  if (IS_IF_PSEUDO(ifp)){
-    IF_PSEUDO_SET(ifp);
-  }
-  else{
-    IF_PSEUDO_UNSET(ifp);
-  }
-#endif /* HAVE_IF_PSEUDO */
-  
   return ifp;
 }
 
@@ -557,7 +598,7 @@ zebra_interface_address_add_read (struct stream *s)
 {
   unsigned int ifindex;
   struct interface *ifp;
-  struct connected *connected;
+  struct connected *ifc;
   struct prefix *p;
   int family;
   int plen;
@@ -574,7 +615,11 @@ zebra_interface_address_add_read (struct stream *s)
     }
 
   /* Allocate new connected address. */
-  connected = connected_new ();
+  ifc = connected_new ();
+  ifc->ifp = ifp;
+
+  /* Fetch flag. */
+  ifc->flags = stream_getc (s);
 
   /* Fetch interface address. */
   p = prefix_new ();
@@ -583,21 +628,21 @@ zebra_interface_address_add_read (struct stream *s)
   plen = prefix_blen (p);
   stream_get (&p->u.prefix, s, plen);
   p->prefixlen = stream_getc (s);
-  connected->address = p;
+  ifc->address = p;
 
   /* Fetch destination address. */
   p = prefix_new ();
   stream_get (&p->u.prefix, s, plen);
   p->family = family;
 
-  connected->destination = p;
+  ifc->destination = p;
 
-  p = connected->address;
+  p = ifc->address;
 
   /* Add connected address to the interface. */
-  connected_add (ifp, connected);
+  listnode_add (ifp->connected, ifc);
 
-  return connected;
+  return ifc;
 }
 
 struct connected *
@@ -610,6 +655,7 @@ zebra_interface_address_delete_read (struct stream *s)
   struct prefix d;
   int family;
   int len;
+  u_char flags;
 
   /* Get interface index. */
   ifindex = stream_getl (s);
@@ -622,8 +668,8 @@ zebra_interface_address_delete_read (struct stream *s)
       return NULL;
     }
 
-  /* Allocate new connected address. */
-  ifc = connected_new ();
+  /* Fetch flag. */
+  flags = stream_getc (s);
 
   /* Fetch interface address. */
   family = p.family = stream_getc (s);
