@@ -44,6 +44,7 @@
 #include "ospfd/ospf_route.h"
 #include "ospfd/ospf_ase.h"
 #include "ospfd/ospf_zebra.h"
+#include "ospfd/ospf_dump.h"
 
 #define DEBUG
 
@@ -66,7 +67,7 @@ ospf_find_asbr_route (struct route_table *rtrs, struct prefix_ipv4 *asbr)
 
   route_unlock_node (rn);
 
-  chosen = list_init ();
+  chosen = list_new ();
 
   /* First try to find intra-area non-bb paths. */
   if (ospf_top->RFC1583Compat == 0)
@@ -75,7 +76,7 @@ ospf_find_asbr_route (struct route_table *rtrs, struct prefix_ipv4 *asbr)
 	if (or->cost < OSPF_LS_INFINITY)
 	  if (!OSPF_IS_AREA_ID_BACKBONE (or->u.std.area_id) &&
 	      or->path_type == OSPF_PATH_INTRA_AREA)
-	    list_add_node (chosen, or);
+	    listnode_add (chosen, or);
 
   /* If none is found -- look through all. */
   if (listcount (chosen) == 0)
@@ -100,7 +101,7 @@ ospf_find_asbr_route (struct route_table *rtrs, struct prefix_ipv4 *asbr)
 	}
 
   if (chosen != rn->info)
-    list_delete_all (chosen);
+    list_delete (chosen);
 
   return best;
 }
@@ -274,7 +275,7 @@ ospf_ase_calculate_new_route (struct ospf_lsa *lsa,
     }
 
   new->type = OSPF_DESTINATION_NETWORK;
-  new->path = list_init ();
+  new->path = list_new ();
   new->u.ext.origin = lsa;
   new->u.ext.tag = ntohl (al->e[0].route_tag);
   new->u.ext.asbr = asbr_route;
@@ -299,6 +300,21 @@ ospf_ase_calculate_route (struct ospf_lsa * lsa, void * p_arg, int n_arg)
   
   assert (lsa);
   al = (struct as_external_lsa *) lsa->data;
+
+#ifdef HAVE_NSSA
+  if (lsa->data->type == OSPF_AS_NSSA_LSA)
+    if (IS_DEBUG_OSPF_NSSA)
+      zlog_info ("ospf_ase_calc(): Processing Type-7");
+
+  /* Stay away from any Local Translated Type-7 LSAs */
+  if (CHECK_FLAG (lsa->flags, OSPF_LSA_LOCAL_XLT))
+    {
+      if (IS_DEBUG_OSPF_NSSA)
+	zlog_info ("ospf_ase_calc(): Rejecting Local Xlt'd");
+      return 0;
+    }
+#endif /* HAVE_NSSA */
+
   zlog_info ("Route[External]: Calculate AS-external-LSA to %s/%d",
 	     inet_ntoa (al->header.id), ip_masklen (al->mask));
   /* (1) If the cost specified by the LSA is LSInfinity, or if the
@@ -517,47 +533,85 @@ ospf_ase_calculate_route (struct ospf_lsa * lsa, void * p_arg, int n_arg)
 }
 
 int
-ospf_ase_compare_tables (struct route_table *new_external_route,
-			 struct route_table *old_external_route)
+ospf_ase_route_match_same (struct route_table *rt, struct prefix *prefix,
+			   struct ospf_route *newor)
 {
   struct route_node *rn;
   struct ospf_route *or;
-  struct ospf_path *path;
-  listnode node;
+  struct ospf_path *op;
+  struct ospf_path *newop;
+  listnode n1;
+  listnode n2;
+
+  if (! rt || ! prefix)
+    return 0;
+
+   rn = route_node_lookup (rt, prefix);
+   if (! rn || ! rn->info)
+     return 0;
+ 
+   route_unlock_node (rn);
+
+   or = rn->info;
+   if (or->path_type != newor->path_type)
+     return 0;
+
+   switch (or->path_type)
+     {
+     case OSPF_PATH_TYPE1_EXTERNAL:
+       if (or->cost != newor->cost)
+	 return 0;
+       break;
+     case OSPF_PATH_TYPE2_EXTERNAL:
+       if ((or->cost != newor->cost) ||
+	   (or->u.ext.type2_cost != newor->u.ext.type2_cost))
+	 return 0;
+       break;
+     default:
+       assert (0);
+       return 0;
+     }
+   
+   if (or->path->count != newor->path->count)
+     return 0;
+       
+   /* Check each path. */
+   for (n1 = listhead (or->path), n2 = listhead (newor->path);
+	n1 && n2; nextnode (n1), nextnode (n2))
+     { 
+       op = getdata (n1);
+       newop = getdata (n2);
+       
+       if (! IPV4_ADDR_SAME (&op->nexthop, &newop->nexthop))
+	 return 0;
+     }
+   return 1;
+}
+
+int
+ospf_ase_compare_tables (struct route_table *new_external_route,
+			 struct route_table *old_external_route)
+{
+  struct route_node *rn, *new_rn;
+  struct ospf_route *or;
   
   /* Remove deleted routes */
   for (rn = route_top (old_external_route); rn; rn = route_next (rn))
     if ((or = rn->info))
-      for (node = listhead (or->path); node; nextnode (node)) 
-	{
-	  path = getdata (node);
-	  
-	  if (path->nexthop.s_addr != INADDR_ANY &&
-	      !ospf_route_match_same (new_external_route,
-				      or->type,
-				      (struct prefix_ipv4 *) &rn->p, 
-				      &path->nexthop))
-	    ospf_zebra_delete ((struct prefix_ipv4 *) &rn->p, 
-			       &path->nexthop);
-	}
-
+      {
+	if (! (new_rn = route_node_lookup (new_external_route, &rn->p)))
+	  ospf_zebra_delete ((struct prefix_ipv4 *) &rn->p, or);
+	else
+	  route_unlock_node (new_rn);
+      }
+  
+	
   /* Install new routes */
   for (rn = route_top (new_external_route); rn; rn = route_next (rn))
     if ((or = rn->info) != NULL)
-      {
-	for (node = listhead (or->path); node; nextnode (node))
-	  {
-	    path = getdata (node);
-	    
-	    if (path->nexthop.s_addr != INADDR_ANY &&
-		!ospf_route_match_same (old_external_route,
-					or->type,
-					(struct prefix_ipv4 *) &rn->p, 
-					&path->nexthop))
-	      ospf_zebra_add ((struct prefix_ipv4 *) &rn->p, &path->nexthop,
-			      or->cost, or);
-	  }
-      }
+      if (! ospf_ase_route_match_same (old_external_route, &rn->p, or))
+	ospf_zebra_add ((struct prefix_ipv4 *) &rn->p, or);
+				       
   return 0;
 }
 
@@ -566,6 +620,11 @@ ospf_ase_calculate_timer (struct thread *t)
 {
   struct ospf *ospf;
 
+#ifdef HAVE_NSSA
+      listnode node;
+      struct ospf_area *area;
+#endif /* HAVE_NSSA */
+
   ospf = THREAD_ARG (t);
   ospf->t_ase_calc = NULL;
 
@@ -577,6 +636,23 @@ ospf_ase_calculate_timer (struct thread *t)
       foreach_lsa (EXTERNAL_LSDB (ospf_top), NULL, 0,
 		   ospf_ase_calculate_route);
 
+#ifdef HAVE_NSSA
+      /*  This version simple adds to the table all NSSA areas  */
+      if (ospf_top->anyNSSA)
+	for (node = listhead (ospf_top->areas); node; nextnode (node))
+	  {
+	    area = getdata (node);
+	    if (IS_DEBUG_OSPF_NSSA)
+	      zlog_info ("ospf_ase_calculate_timer(): looking at area %s",
+			 inet_ntoa (area->area_id));
+
+	    if (area->external_routing == OSPF_AREA_NSSA)
+
+	      foreach_lsa (NSSA_LSDB (area), NULL, 0,
+			   ospf_ase_calculate_route);
+	  }
+#endif /* HAVE_NSSA */
+
       /* Compare old and new external routing table and install the
 	 difference info zebra/kernel */
       ospf_ase_compare_tables (ospf_top->new_external_route,
@@ -626,12 +702,12 @@ ospf_ase_register_external_lsa (struct ospf_lsa *lsa, struct ospf *top)
 
   rn = route_node_get (top->external_lsas, (struct prefix *) &p);
   if ((lst = rn->info) == NULL)
-    rn->info = lst = list_init();
+    rn->info = lst = list_new();
 
   /* We assume that if LSA is deleted from DB
      is is also deleted from this RT */
 
-  list_add_node (lst, ospf_lsa_lock (lsa));
+  listnode_add (lst, ospf_lsa_lock (lsa));
 }
 
 void
@@ -652,7 +728,7 @@ ospf_ase_unregister_external_lsa (struct ospf_lsa *lsa, struct ospf *top)
   lst = rn->info;
   assert (lst);
 
-  list_delete_by_val (lst, lsa);
+  listnode_delete (lst, lsa);
   ospf_lsa_unlock (lsa);
 }
 
@@ -670,7 +746,7 @@ ospf_ase_external_lsas_finish (struct route_table *rt)
 	for (node = listhead (lst); node; node = nextnode (node))
 	  if ((lsa = getdata (node)) != NULL)
 	    ospf_lsa_unlock (lsa);
-	list_delete_all (lst);
+	list_delete (lst);
       }
   
   route_table_finish (rt);
