@@ -46,9 +46,9 @@
 #include "ospfd/ospf_abr.h"
 #include "ospfd/ospf_route.h"
 #include "ospfd/ospf_zebra.h"
+#include "ospfd/ospf_dump.h"
 
 extern struct zclient *zclient;
-
 
 /* Do the LSA acking specified in table 19, Section 13.5, row 2
  * This get called from ospf_flood_out_interface. Declared inline 
@@ -72,7 +72,7 @@ ospf_flood_delayed_lsa_ack (struct ospf_neighbor *inbr, struct ospf_lsa *lsa)
     return;
 
   /* Schedule a delayed LSA Ack to be sent */ 
-  list_add_node (inbr->oi->ls_ack, ospf_lsa_lock (lsa));
+  listnode_add (inbr->oi->ls_ack, ospf_lsa_lock (lsa));
 }
 
 /* Check LSA is related to external info. */
@@ -91,17 +91,21 @@ ospf_external_info_check (struct ospf_lsa *lsa)
   p.prefixlen = ip_masklen (al->mask);
 
   for (type = 0; type <= ZEBRA_ROUTE_MAX; type++)
-    if (ospf_is_type_redistributed (type))
-      if (EXTERNAL_INFO (type))
-	{
-	  rn = route_node_lookup (EXTERNAL_INFO (type), (struct prefix *) &p);
-	  if (rn != NULL)
-	    {
-	      route_unlock_node (rn);
-	      if (rn->info != NULL)
-		return (struct external_info *) rn->info;
-	    }
-	}
+    {
+      int redist_type = is_prefix_default (&p) ? DEFAULT_ROUTE : type;
+      if (ospf_is_type_redistributed (redist_type))
+	if (EXTERNAL_INFO (type))
+	  {
+	    rn = route_node_lookup (EXTERNAL_INFO (type),
+				    (struct prefix *) &p);
+	    if (rn != NULL)
+	      {
+		route_unlock_node (rn);
+		if (rn->info != NULL)
+		  return (struct external_info *) rn->info;
+	      }
+	  }
+    }
 
   return NULL;
 }
@@ -115,8 +119,9 @@ ospf_process_self_originated_lsa (struct ospf_lsa *new, struct ospf_area *area)
   struct interface *ifp;
   struct external_info *ei;
   
-  zlog_info ("LSA[Type%d:%s]: Process self-originated LSA",
-	     new->data->type, inet_ntoa (new->data->id));
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info ("LSA[Type%d:%s]: Process self-originated LSA",
+	       new->data->type, inet_ntoa (new->data->id));
 
   /* If we're here, we installed a self-originated LSA that we received
      from a neighbor, i.e. it's more recent.  We must see whether we want
@@ -170,16 +175,44 @@ ospf_process_self_originated_lsa (struct ospf_lsa *new, struct ospf_area *area)
       ospf_schedule_abr_task ();
       break;
     case OSPF_AS_EXTERNAL_LSA :
+#ifdef HAVE_NSSA
+    case OSPF_AS_NSSA_LSA:
+#endif /* HAVE_NSSA */
       ei = ospf_external_info_check (new);
       if (ei)
 	ospf_external_lsa_refresh (new, ei, LSA_REFRESH_FORCE);
       else
 	ospf_lsa_flush_as (new);
       break;
+    default:
+      break;
     }
 }
 
 /* OSPF LSA flooding -- RFC2328 Section 13.(5). */
+
+/* Now Updated for NSSA operation, as follows:
+
+
+	Type-5's have no change.  Blocked to STUB or NSSA.
+
+	Type-7's can be received, and if a DR
+	they will also flood the local NSSA Area as Type-7's
+
+	If a Self-Originated LSA (now an ASBR), 
+	The LSDB will be updated as Type-5's, (for continual re-fresh)
+
+	    If an NSSA-IR it is installed/flooded as Type-7, P-bit on.
+	    if an NSSA-ABR it is installed/flooded as Type-7, P-bit off.
+
+	Later, during the ABR TASK, if the ABR is the Elected NSSA
+	translator, then All Type-7s (with P-bit ON) are Translated to
+	Type-5's and flooded to all non-NSSA/STUB areas.
+
+	During ASE Calculations, 
+	    non-ABRs calculate external routes from Type-7's
+	    ABRs calculate external routes from Type-5's and non-self Type-7s
+*/
 int
 ospf_flood (struct ospf_neighbor *nbr, struct ospf_lsa *current,
 	    struct ospf_lsa *new)
@@ -188,7 +221,11 @@ ospf_flood (struct ospf_neighbor *nbr, struct ospf_lsa *current,
   struct timeval now;
   int lsa_ack_flag;
 
-  zlog_info ("LSA[Flooding]: start");
+  /* Type-7 LSA's will be flooded throughout their native NSSA area,
+     but will also be flooded as Type-5's into ABR capable links.  */
+
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info ("LSA[Flooding]: start");
 
   lsa_ack_flag = 0;
   oi = nbr->oi;
@@ -205,7 +242,8 @@ ospf_flood (struct ospf_neighbor *nbr, struct ospf_lsa *current,
       tv_cmp (tv_sub (now, current->tv_recv),
 	      int2tv (OSPF_MIN_LS_ARRIVAL)) < 0)
     {
-      zlog_info ("LSA[Flooding]: LSA is received recently.");
+      if (IS_DEBUG_OSPF_EVENT)
+	zlog_info ("LSA[Flooding]: LSA is received recently.");
       return -1;
     }
 
@@ -216,11 +254,12 @@ ospf_flood (struct ospf_neighbor *nbr, struct ospf_lsa *current,
      interface. */
   lsa_ack_flag = ospf_flood_through (nbr, new);
 
-  /* Remove the current database copy from all neighbors'
-     Link state retransmission lists. */
+  /* Remove the current database copy from all neighbors' Link state
+     retransmission lists.  Only AS_EXTERNAL does not have area ID.
+     All other (even NSSA's) do have area ID.  */
   if (current)
     {
-      if(current->data->type != OSPF_AS_EXTERNAL_LSA) 
+      if (current->data->type != OSPF_AS_EXTERNAL_LSA)
 	ospf_ls_retransmit_delete_nbr_all (nbr->oi->area, current);
       else
 	ospf_ls_retransmit_delete_nbr_all (NULL, current);
@@ -238,8 +277,17 @@ ospf_flood (struct ospf_neighbor *nbr, struct ospf_lsa *current,
   */
   SET_FLAG (new->flags, OSPF_LSA_RECEIVED);
   ospf_lsa_is_self_originated (new); /* Let it set the flag */
+
   new = ospf_lsa_install (nbr->oi, new);
 
+#ifdef HAVE_NSSA 
+  if (IS_DEBUG_OSPF_NSSA)
+    zlog_info ("LSA[Flooding]: Type-%d installed", new->data->type);
+
+  /* if  (new->data->type == OSPF_AS_NSSA_LSA )
+     return 0;  */
+#endif /* HAVE_NSSA */
+
   /* Acknowledge the receipt of the LSA by sending a Link State
      Acknowledgment packet back out the receiving interface. */
   if (lsa_ack_flag)
@@ -269,7 +317,7 @@ ospf_flood_through_interface (struct interface *ifp,
   struct route_node *rn;
   int retx_flag;
 
-  if (ospf_zlog)
+  if (IS_DEBUG_OSPF_EVENT)
     zlog_info ("ospf_flood_through_interface(): considering int %s",
 	       ifp->name); 
 
@@ -290,7 +338,7 @@ ospf_flood_through_interface (struct interface *ifp,
 	continue;
 
       onbr = rn->info;
-      if (ospf_zlog)
+      if (IS_DEBUG_OSPF_EVENT)
 	zlog_info ("ospf_flood_through_interface(): considering nbr %s",
 		   inet_ntoa (onbr->router_id));
 
@@ -308,7 +356,7 @@ ospf_flood_through_interface (struct interface *ifp,
 	 already.  Compare the new LSA to the neighbor's copy: */
       if (onbr->status < NSM_Full)
 	{
-	  if (ospf_zlog)
+	  if (IS_DEBUG_OSPF_EVENT)
 	    zlog_info ("ospf_flood_through_interface(): nbr adj is not Full");
 	  ls_req = ospf_ls_request_lookup (onbr, lsa);
 	  if (ls_req != NULL)
@@ -369,6 +417,11 @@ ospf_flood_through_interface (struct interface *ifp,
 	 received the LSA already. */
       if (NBR_IS_DR (inbr) || NBR_IS_BDR (inbr))
 	{
+#ifdef HAVE_NSSA
+	  if (IS_DEBUG_OSPF_NSSA)
+	    zlog_info ("ospf_flood_through_interface(): "
+		       "DR/BDR NOT SEND to int %s", oi->ifp->name);
+#endif /* HAVE_NSSA */
 	  return 1;
 	}
 	  
@@ -377,8 +430,14 @@ ospf_flood_through_interface (struct interface *ifp,
 	 Designated Router will do the flooding on this interface.
 	 However, if the Designated Router fails the router will
 	 end up retransmitting the updates. */
+
       if (oi->status == ISM_Backup)
 	{
+#ifdef HAVE_NSSA
+	  if (IS_DEBUG_OSPF_NSSA)
+	    zlog_info ("ospf_flood_through_interface(): "
+		       "ISM_Backup NOT SEND to int %s", oi->ifp->name);
+#endif /* HAVE_NSSA */
 	  return 1;
 	}
     }
@@ -390,9 +449,17 @@ ospf_flood_through_interface (struct interface *ifp,
      State Update packet (until the LS age field reaches the maximum
      value of MaxAge). */
 
-  if (ospf_zlog)
+#ifdef HAVE_NSSA
+  if (IS_DEBUG_OSPF_NSSA)
+    zlog_info ("ospf_flood_through_interface(): "
+	       "DR/BDR sending upd to int %s", oi->ifp->name);
+#else /* ! HAVE_NSSA */
+
+  if (IS_DEBUG_OSPF_EVENT)
     zlog_info ("ospf_flood_through_interface(): "
 	       "sending upd to int %s", oi->ifp->name);
+#endif /* HAVE_NSSA */
+
   /*  RFC2328  Section 13.3
       On non-broadcast networks, separate	Link State Update
       packets must be sent, as unicasts, to each adjacent	neighbor
@@ -450,10 +517,23 @@ ospf_flood_through_as (struct ospf_neighbor *inbr, struct ospf_lsa *lsa)
 
   lsa_ack_flag = 0;
 
-  /* AS-external-LSAs are flooded throughout the entire AS, with the
+  /* The incoming LSA is type 5 or type 7  (AS-EXTERNAL or AS-NSSA )
+
+    Divert the Type-5 LSA's to all non-NSSA/STUB areas
+
+    Divert the Type-7 LSA's to all NSSA areas
+
+     AS-external-LSAs are flooded throughout the entire AS, with the
      exception of stub areas (see Section 3.6).  The eligible
      interfaces are all the router's interfaces, excluding virtual
      links and those interfaces attaching to stub areas.  */
+
+#ifdef HAVE_NSSA
+  if (CHECK_FLAG (lsa->flags, OSPF_LSA_LOCAL_XLT)) /* Translated from 7  */
+    if (IS_DEBUG_OSPF_NSSA)
+      zlog_info ("Flood/AS: NSSA TRANSLATED LSA");
+#endif /* HAVE_NSSA */
+
   for (node = listhead (ospf_top->areas); node; nextnode (node))
     {
       int continue_flag = 0;
@@ -463,21 +543,42 @@ ospf_flood_through_as (struct ospf_neighbor *inbr, struct ospf_lsa *lsa)
       switch (area->external_routing)
 	{
 	  /* Don't send AS externals into stub areas.  Various types
-             of support for partial stub areas can be implemented here. */
+             of support for partial stub areas can be implemented
+             here.  NSSA's will receive Type-7's that have areas
+             matching the originl LSA. */
+	case OSPF_AREA_NSSA:	/* Sending Type 5 or 7 into NSSA area */
+#ifdef HAVE_NSSA
+	  /* Type-7, flood NSSA area */
+          if (lsa->data->type == OSPF_AS_NSSA_LSA) 
+	    /* We will send it. */
+	    continue_flag = 0;
+          else
+	    continue_flag = 1;  /* Skip this NSSA area for Type-5's et al */
+          break;
+#endif /* HAVE_NSSA */
+
 	case OSPF_AREA_TYPE_MAX:
-	case OSPF_AREA_NSSA:
 	case OSPF_AREA_STUB:
-	  continue_flag = 1;
+	  continue_flag = 1;	/* Skip this area. */
 	  break;
+
 	case OSPF_AREA_DEFAULT:
 	default:
-	  continue_flag = 0;
+#ifdef HAVE_NSSA
+	  /* No Type-7 into normal area */
+          if (lsa->data->type == OSPF_AS_NSSA_LSA) 
+	    continue_flag = 1; /* skip Type-7 */
+          else
+#endif /* HAVE_NSSA */
+	    continue_flag = 0;	/* Do this area. */
 	  break;
 	}
       
       /* Do continue for above switch.  Saves a big if then mess */
       if (continue_flag) 
-	continue;
+	continue; /* main for-loop */
+      
+      /* send to every interface in this area */
 
       for (if_node = listhead (area->iflist); if_node; nextnode (if_node))
 	{
@@ -486,20 +587,27 @@ ospf_flood_through_as (struct ospf_neighbor *inbr, struct ospf_lsa *lsa)
 
 	  /* Skip virtual links */
 	  if (oi->type !=  OSPF_IFTYPE_VIRTUALLINK)
-	    if (ospf_flood_through_interface (ifp, inbr, lsa))
+	    if (ospf_flood_through_interface (ifp, inbr, lsa)) /* lsa */
 	      lsa_ack_flag = 1;
 	}
-    }
+    } /* main area for-loop */
   
   return (lsa_ack_flag);
 }
 
-
 int
 ospf_flood_through (struct ospf_neighbor *inbr, struct ospf_lsa *lsa)
 {
   int lsa_ack_flag = 0;
   
+  /* Type-7 LSA's for NSSA are flooded throughout the AS here, and
+     upon return are updated in the LSDB for Type-7's.  Later,
+     re-fresh will re-send them (and also, if ABR, packet code will
+     translate to Type-5's)
+  
+     As usual, Type-5 LSA's (if not DISCARDED because we are STUB or
+     NSSA) are flooded throughout the AS, and are updated in the
+     global table.  */
   switch (lsa->data->type)
     {
     case OSPF_ROUTER_LSA:
@@ -508,9 +616,19 @@ ospf_flood_through (struct ospf_neighbor *inbr, struct ospf_lsa *lsa)
     case OSPF_SUMMARY_LSA_ASBR:
       lsa_ack_flag = ospf_flood_through_area (inbr->oi->area, inbr, lsa);
       break;
-    case OSPF_AS_EXTERNAL_LSA:
+    case OSPF_AS_EXTERNAL_LSA: /* Type-5 */
       lsa_ack_flag = ospf_flood_through_as (inbr, lsa);
       break;
+#ifdef HAVE_NSSA
+      /* Type-7 Only received within NSSA, then flooded */
+    case OSPF_AS_NSSA_LSA:
+      /* Any P-bit was installed with the Type-7. */
+      lsa_ack_flag = ospf_flood_through_area (inbr->oi->area, inbr, lsa);
+
+      if (IS_DEBUG_OSPF_NSSA)
+	zlog_info ("ospf_flood_through: LOCAL NSSA FLOOD of Type-7.");
+      break;
+#endif /* HAVE_NSSA */
     default:
       break;
     }
@@ -596,11 +714,27 @@ ospf_ls_retransmit_isempty (struct ospf_neighbor *nbr)
 void
 ospf_ls_retransmit_add (struct ospf_neighbor *nbr, struct ospf_lsa *lsa)
 {
-  if (!ospf_ls_retransmit_lookup (nbr, lsa))
+  struct ospf_lsa *old;
+
+  old = ospf_ls_retransmit_lookup (nbr, lsa);
+
+  if (ospf_lsa_more_recent (old, lsa) < 0)
+    {
+      if (old)
+	{
+	  old->retransmit_counter--;
+	  new_lsdb_delete (&nbr->ls_rxmt, old);
+	}
+      lsa->retransmit_counter++;
+      new_lsdb_add (&nbr->ls_rxmt, lsa);
+    }
+/*
+    if (!ospf_ls_retransmit_lookup (nbr, lsa))
     {
       lsa->retransmit_counter++;
       new_lsdb_add (&nbr->ls_rxmt, lsa);
     }
+*/    
 }
 
 /* Remove LSA from neibghbor's ls-retransmit list. */
@@ -645,15 +779,6 @@ ospf_ls_retransmit_lookup (struct ospf_neighbor *nbr, struct ospf_lsa *lsa)
   return new_lsdb_lookup (&nbr->ls_rxmt, lsa);
 }
 
-#if 0  /* Probably, this function will not be used -- takada. */
-/* Remove a neighbor's Link State Retransmit list. */
-void
-ospf_ls_retransmit_delete_all (struct ospf_neighbor *nbr)
-{
-  new_lsdb_delete_all (&nbr->ls_rxmt);
-}
-#endif
-
 /* Remove All neighbor/interface's Link State Retransmit list in area. */
 void
 ospf_ls_retransmit_delete_nbr_all (struct ospf_area *area,
