@@ -38,6 +38,7 @@
 #include "ospfd/ospf_route.h"
 #include "ospfd/ospf_spf.h"
 #include "ospfd/ospf_zebra.h"
+#include "ospfd/ospf_dump.h"
 
 #ifndef NEW_OSPF_ROUTE
 struct ospf_route *
@@ -64,7 +65,7 @@ ospf_route_free (struct ospf_route *or)
       for (node = listhead (or->path); node; nextnode (node))
 	ospf_path_free (node->data);
 
-      list_delete_all (or->path);
+      list_delete (or->path);
     }
 
   XFREE (MTYPE_OSPF_ROUTE, or);
@@ -103,21 +104,13 @@ ospf_route_delete (struct route_table *rt)
 {
   struct route_node *rn;
   struct ospf_route *or;
-  struct ospf_path *path;
-  listnode pnode;
 
   for (rn = route_top (rt); rn; rn = route_next (rn))
     if ((or = rn->info) != NULL)
       {
 	if (or->type == OSPF_DESTINATION_NETWORK)
-	  for (pnode = listhead (or->path); pnode; nextnode (pnode))
-	    {
-	      path = getdata (pnode);
-
-	      if (path->nexthop.s_addr != INADDR_ANY)
-		ospf_zebra_delete ((struct prefix_ipv4 *) &rn->p,
-				   &path->nexthop);
-	    }
+	  ospf_zebra_delete ((struct prefix_ipv4 *) &rn->p,
+				       or);
 	else if (or->type == OSPF_DESTINATION_DISCARD)
 	  ospf_zebra_delete_discard ((struct prefix_ipv4 *) &rn->p);
       }
@@ -179,43 +172,48 @@ ospf_route_table_free (struct route_table *rt)
 /* If a prefix and a nexthop match any route in the routing table,
    then return 1, otherwise return 0. */
 int
-ospf_route_match_same (struct route_table *rt, int type,
-		       struct prefix_ipv4 *prefix, struct in_addr *nexthop)
+ospf_route_match_same (struct route_table *rt, struct prefix_ipv4 *prefix,
+		       struct ospf_route *newor)
 {
   struct route_node *rn;
   struct ospf_route *or;
   struct ospf_path *op;
-  listnode node;
+  struct ospf_path *newop;
+  listnode n1;
+  listnode n2;
 
-  if (!rt || !prefix)
+  if (! rt || ! prefix)
     return 0;
 
    rn = route_node_lookup (rt, (struct prefix *) prefix);
-   if (!rn || !rn->info)
+   if (! rn || ! rn->info)
      return 0;
  
+   route_unlock_node (rn);
+
    or = rn->info;
-   if (or->type == type)
+   if (or->type == newor->type && or->cost == newor->cost)
      {
        if (or->type == OSPF_DESTINATION_NETWORK)
-	 for (node = listhead (or->path); node; nextnode (node))
-	   { 
-	     op = getdata (node);
+	 {
+	   if (or->path->count != newor->path->count)
+	     return 0;
 
-	     if (op->nexthop.s_addr != INADDR_ANY && nexthop &&
-		 IPV4_ADDR_SAME (&op->nexthop, nexthop))
-	       {
-		 route_unlock_node (rn);
-		 return 1;
-	       }
-	   }
+	   /* Check each path. */
+	   for (n1 = listhead (or->path), n2 = listhead (newor->path);
+		n1 && n2; nextnode (n1), nextnode (n2))
+	     { 
+	       op = getdata (n1);
+	       newop = getdata (n2);
+
+	       if (! IPV4_ADDR_SAME (&op->nexthop, &newop->nexthop))
+		 return 0;
+	     }
+	   return 1;
+	 }
        else if (prefix_same (&rn->p, (struct prefix *) prefix))
-         {
-           route_unlock_node (rn);
-           return 1;
-         }
+	 return 1;
      }
-
   return 0;
 }
 
@@ -225,31 +223,22 @@ ospf_route_delete_uniq (struct route_table *rt, struct route_table *cmprt)
 {
   struct route_node *rn;
   struct ospf_route *or;
-  struct ospf_path *path;
-  listnode node;
 
   for (rn = route_top (rt); rn; rn = route_next (rn))
     if ((or = rn->info) != NULL) 
       if (or->path_type == OSPF_PATH_INTRA_AREA ||
 	  or->path_type == OSPF_PATH_INTER_AREA)
 	{
-
-	if (or->type == OSPF_DESTINATION_NETWORK)
-	  for (node = listhead (or->path); node; nextnode (node)) 
+	  if (or->type == OSPF_DESTINATION_NETWORK)
 	    {
-	      path = getdata (node);
-
-	      if (path->nexthop.s_addr != INADDR_ANY &&
-		  !ospf_route_match_same (cmprt, or->type,
-					  (struct prefix_ipv4 *) &rn->p, 
-					  &path->nexthop))
-		ospf_zebra_delete ((struct prefix_ipv4 *) &rn->p, 
-				   &path->nexthop);
+	      if (! ospf_route_match_same (cmprt, 
+					   (struct prefix_ipv4 *) &rn->p, or))
+		ospf_zebra_delete ((struct prefix_ipv4 *) &rn->p, or);
 	    }
-	else if (or->type == OSPF_DESTINATION_DISCARD)
-	  if (!ospf_route_match_same (cmprt, or->type,
-				      (struct prefix_ipv4 *) &rn->p, 0))
-	    ospf_zebra_delete_discard ((struct prefix_ipv4 *) &rn->p);
+	  else if (or->type == OSPF_DESTINATION_DISCARD)
+	    if (! ospf_route_match_same (cmprt,
+					 (struct prefix_ipv4 *) &rn->p, or))
+	      ospf_zebra_delete_discard ((struct prefix_ipv4 *) &rn->p);
 	}
 }
 
@@ -259,8 +248,6 @@ ospf_route_install (struct route_table *rt)
 {
   struct route_node *rn;
   struct ospf_route *or;
-  struct ospf_path *path;
-  listnode node;
 
   /* rt contains new routing table, new_table contains an old one.
      updating pointers */
@@ -279,20 +266,14 @@ ospf_route_install (struct route_table *rt)
     if ((or = rn->info) != NULL)
       {
 	if (or->type == OSPF_DESTINATION_NETWORK)
-	  for (node = listhead (or->path); node; nextnode (node))
-	    {
-	      path = getdata (node);
-
-	      if (path->nexthop.s_addr != INADDR_ANY &&
-		  ! ospf_route_match_same (ospf_top->old_table, or->type,
-					   (struct prefix_ipv4 *) &rn->p, 
-					   &path->nexthop))
-		ospf_zebra_add ((struct prefix_ipv4 *) &rn->p, &path->nexthop,
-				or->cost, or);
-	    }
+	  {
+	    if (! ospf_route_match_same (ospf_top->old_table,
+					 (struct prefix_ipv4 *)&rn->p, or))
+	      ospf_zebra_add ((struct prefix_ipv4 *) &rn->p, or);
+	  }
 	else if (or->type == OSPF_DESTINATION_DISCARD)
-	  if (!ospf_route_match_same (ospf_top->old_table, or->type,
-				      (struct prefix_ipv4 *) &rn->p, 0))
+	  if (! ospf_route_match_same (ospf_top->old_table,
+				       (struct prefix_ipv4 *) &rn->p, or))
 	    ospf_zebra_add_discard ((struct prefix_ipv4 *) &rn->p);
       }
 }
@@ -332,14 +313,14 @@ ospf_intra_route_add (struct route_table *rt, struct vertex *v,
   if (v->type == OSPF_VERTEX_NETWORK)
     {
       or->type = OSPF_DESTINATION_NETWORK;
-      or->path = list_init ();
+      or->path = list_new ();
 
       for (nnode = listhead (v->nexthop); nnode; nextnode (nnode))
 	{
 	  nexthop = getdata (nnode);
 	  path = ospf_path_new ();
 	  path->nexthop = nexthop->router;
-	  list_add_node (or->path, path);
+	  listnode_add (or->path, path);
 	}
     }
   else
@@ -347,6 +328,9 @@ ospf_intra_route_add (struct route_table *rt, struct vertex *v,
 
   or->id = v->id;
   or->u.std.area_id = area->area_id;
+#ifdef HAVE_NSSA
+  or->u.std.external_routing= area->external_routing;
+#endif /* HAVE_NSSA */
   or->path_type = OSPF_PATH_INTRA_AREA;
   or->cost = v->distance;
 
@@ -363,12 +347,14 @@ ospf_intra_add_router (struct route_table *rt, struct vertex *v,
   struct prefix_ipv4 p;
   struct router_lsa *lsa;
 
-  zlog_info ("ospf_intra_add_router: Start");
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info ("ospf_intra_add_router: Start");
 
   lsa = (struct router_lsa *) v->lsa;
 
-  zlog_info ("ospf_intra_add_router: LS ID: %s",
-	     inet_ntoa (lsa->header.id));
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info ("ospf_intra_add_router: LS ID: %s",
+	       inet_ntoa (lsa->header.id));
 
   ospf_vl_up_check (area, lsa->header.id, v);
 
@@ -380,8 +366,9 @@ ospf_intra_add_router (struct route_table *rt, struct vertex *v,
      "router". */
   if (! IS_ROUTER_LSA_BORDER (lsa) && ! IS_ROUTER_LSA_EXTERNAL (lsa))
     {
-      zlog_info ("ospf_intra_add_router: "
-		 "this router is neither ASBR nor ABR, skipping it");
+      if (IS_DEBUG_OSPF_EVENT)
+	zlog_info ("ospf_intra_add_router: "
+		   "this router is neither ASBR nor ABR, skipping it");
       return;
     }
 
@@ -398,6 +385,9 @@ ospf_intra_add_router (struct route_table *rt, struct vertex *v,
 
   or->id = v->id;
   or->u.std.area_id = area->area_id;
+#ifdef HAVE_NSSA
+  or->u.std.external_routing = area->external_routing;
+#endif /* HAVE_NSSA */
   or->path_type = OSPF_PATH_INTRA_AREA;
   or->cost = v->distance;
   or->type = OSPF_DESTINATION_ROUTER;
@@ -420,20 +410,21 @@ ospf_intra_add_router (struct route_table *rt, struct vertex *v,
   p.prefix = v->id;
   p.prefixlen = IPV4_MAX_BITLEN;
 
-  zlog_info ("ospf_intra_add_router: talking about %s/%d",
-	     inet_ntoa (p.prefix), p.prefixlen);
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info ("ospf_intra_add_router: talking about %s/%d",
+	       inet_ntoa (p.prefix), p.prefixlen);
 
   rn = route_node_get (rt, (struct prefix *) &p);
 
   /* Note that we keep all routes to ABRs and ASBRs, not only the best */
   if (rn->info == NULL)
-    rn->info = list_init ();
+    rn->info = list_new ();
   else
     route_unlock_node (rn);
 
   ospf_route_copy_nexthops_from_vertex (or, v);
 
-  list_add_node (rn->info, or);
+  listnode_add (rn->info, or);
 
   zlog_info ("ospf_intra_add_router: Start");
 }
@@ -488,6 +479,9 @@ ospf_intra_add_transit (struct route_table *rt, struct vertex *v,
 
   or->id = v->id;
   or->u.std.area_id = area->area_id;
+#ifdef HAVE_NSSA
+  or->u.std.external_routing = area->external_routing;
+#endif /* HAVE_NSSA */
   or->path_type = OSPF_PATH_INTRA_AREA;
   or->cost = v->distance;
   or->type = OSPF_DESTINATION_NETWORK;
@@ -511,7 +505,8 @@ ospf_intra_add_stub (struct route_table *rt, struct router_lsa_link *link,
   struct ospf_interface *oi;
   struct ospf_path *path;
 
-  zlog_info ("ospf_intra_add_stub(): Start");
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info ("ospf_intra_add_stub(): Start");
 
   lsa = (struct router_lsa *) v->lsa;
 
@@ -520,8 +515,9 @@ ospf_intra_add_stub (struct route_table *rt, struct router_lsa_link *link,
   p.prefixlen = ip_masklen (link->link_data);
   apply_mask_ipv4 (&p);
 
-  zlog_info ("ospf_intra_add_stub(): processing route to %s/%d",  
-	     inet_ntoa (p.prefix), p.prefixlen);
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info ("ospf_intra_add_stub(): processing route to %s/%d",  
+	       inet_ntoa (p.prefix), p.prefixlen);
 
   /* (1) Calculate the distance D of stub network from the root.  D is
      equal to the distance from the root to the router vertex
@@ -529,8 +525,9 @@ ospf_intra_add_stub (struct route_table *rt, struct router_lsa_link *link,
      cost. */
   cost = v->distance + ntohs (link->m[0].metric);
 
-  zlog_info ("ospf_intra_add_stub(): calculated cost is %d + %d = %d", 
-	     v->distance, ntohs(link->m[0].metric), cost);
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info ("ospf_intra_add_stub(): calculated cost is %d + %d = %d", 
+	       v->distance, ntohs(link->m[0].metric), cost);
 
   rn = route_node_get (rt, (struct prefix *) &p);
 
@@ -543,8 +540,9 @@ ospf_intra_add_stub (struct route_table *rt, struct router_lsa_link *link,
 
       cur_or = rn->info;
 
-      zlog_info ("ospf_intra_add_stub(): "
-		 "another route to the same prefix found");
+      if (IS_DEBUG_OSPF_EVENT)
+	zlog_info ("ospf_intra_add_stub(): "
+		   "another route to the same prefix found");
 
       /* Compare this distance to the current best cost to the stub
 	 network.  This is done by looking up the stub network's
@@ -553,7 +551,8 @@ ospf_intra_add_stub (struct route_table *rt, struct router_lsa_link *link,
 	 LSA. */
       if (cost > cur_or->cost)
 	{
-	  zlog_info ("ospf_intra_add_stub(): old route is better, exit");
+	  if (IS_DEBUG_OSPF_EVENT)
+	    zlog_info ("ospf_intra_add_stub(): old route is better, exit");
 	  return;
 	}
 
@@ -572,7 +571,8 @@ ospf_intra_add_stub (struct route_table *rt, struct router_lsa_link *link,
 
       if (cost == cur_or->cost)
 	{
-	  zlog_info ("ospf_intra_add_stub(): routes are equal, merge");
+	  if (IS_DEBUG_OSPF_EVENT)
+	    zlog_info ("ospf_intra_add_stub(): routes are equal, merge");
 
 	  ospf_route_copy_nexthops_from_vertex (cur_or, v);
 
@@ -590,11 +590,12 @@ ospf_intra_add_stub (struct route_table *rt, struct router_lsa_link *link,
 
       if (cost < cur_or->cost)
 	{
-	  zlog_info ("ospf_intra_add_stub(): new route is better, set it");
+	  if (IS_DEBUG_OSPF_EVENT)
+	    zlog_info ("ospf_intra_add_stub(): new route is better, set it");
 
 	  cur_or->cost = cost;
 
-	  list_delete_all (cur_or->path);
+	  list_delete (cur_or->path);
 	  cur_or->path = NULL;
 
 	  ospf_route_copy_nexthops_from_vertex (cur_or, v);
@@ -604,45 +605,56 @@ ospf_intra_add_stub (struct route_table *rt, struct router_lsa_link *link,
 	}
     }
 
-  zlog_info ("ospf_intra_add_stub(): installing new route");
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info ("ospf_intra_add_stub(): installing new route");
 
   or = ospf_route_new ();
 
   or->id = v->id;
   or->u.std.area_id = area->area_id;
+#ifdef HAVE_NSSA
+  or->u.std.external_routing = area->external_routing;
+#endif /* HAVE_NSSA */
   or->path_type = OSPF_PATH_INTRA_AREA;
   or->cost = cost;
   or->type = OSPF_DESTINATION_NETWORK;
   or->u.std.origin = (struct lsa_header *) lsa;
-  or->path = list_init ();
+  or->path = list_new ();
 
   /* Nexthop is depend on connection type. */
   if (v != area->spf)
     {
-      zlog_info ("ospf_intra_add_stub(): this network is on remote router");
+      if (IS_DEBUG_OSPF_EVENT)
+	zlog_info ("ospf_intra_add_stub(): this network is on remote router");
       ospf_route_copy_nexthops_from_vertex (or, v);
     }
   else
     {
-      zlog_info ("ospf_intra_add_stub(): this network is on this router");
+      if (IS_DEBUG_OSPF_EVENT)
+	zlog_info ("ospf_intra_add_stub(): this network is on this router");
 
       if ((oi = ospf_if_lookup_by_prefix (&p)))
 	{
-	  zlog_info ("ospf_intra_add_stub(): the interface is %s",
-		     oi->ifp->name);
+	  if (IS_DEBUG_OSPF_EVENT)
+	    zlog_info ("ospf_intra_add_stub(): the interface is %s",
+		       oi->ifp->name);
 
 	  path = ospf_path_new ();
 	  path->nexthop.s_addr = 0;
 	  path->ifp = oi->ifp;
-	  list_add_node (or->path, path);
+	  listnode_add (or->path, path);
 	}
       else
-	zlog_info ("ospf_intra_add_stub(): where's the interface ?");
+	{
+	  if (IS_DEBUG_OSPF_EVENT)
+	    zlog_info ("ospf_intra_add_stub(): where's the interface ?");
+	}
     }
 
   rn->info = or;
 
-  zlog_info("ospf_intra_add_stub(): Stop");
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info("ospf_intra_add_stub(): Stop");
 }
 
 char *ospf_path_type_str[] =
@@ -734,13 +746,21 @@ show_ip_ospf_route_network (struct vty *vty, struct route_table *rt)
 	  case OSPF_PATH_INTER_AREA:
 	    if (or->type == OSPF_DESTINATION_NETWORK)
 	      vty_out (vty, "N IA %-18s    [%d] area: %s%s", buf1, or->cost,
-		       inet_ntoa (or->u.std.area_id), VTY_NEWLINE);
+#ifdef HAVE_NSSA
+		       ait_ntoa (or->u.std.area_id, or->u.std.external_routing), VTY_NEWLINE);
+#else /* HAVE_NSSA */
+		     inet_ntoa (or->u.std.area_id), VTY_NEWLINE);
+#endif /* HAVE_NSSA */
 	    else if (or->type == OSPF_DESTINATION_DISCARD)
 	      vty_out (vty, "D IA %-18s    Discard entry%s", buf1, VTY_NEWLINE);
 	    break;
 	  case OSPF_PATH_INTRA_AREA:
 	    vty_out (vty, "N    %-18s    [%d] area: %s%s", buf1, or->cost,
+#ifdef HAVE_NSSA
+		     ait_ntoa (or->u.std.area_id, or->u.std.external_routing), VTY_NEWLINE);
+#else /* HAVE_NSSA */
 		     inet_ntoa (or->u.std.area_id), VTY_NEWLINE);
+#endif /* HAVE_NSSA */
 	    break;
 	  default:
 	    break;
@@ -791,7 +811,11 @@ show_ip_ospf_route_router (struct vty *vty, struct route_table *rtrs)
 	      /* Show path. */
 	      vty_out (vty, "%s [%d] area: %s",
 		       (or->path_type == OSPF_PATH_INTER_AREA ? "IA" : "  "),
+#ifdef HAVE_NSSA
+		       or->cost, ait_ntoa (or->u.std.area_id,or->u.std.external_routing));
+#else /* HAVE_NSSA */
 		       or->cost, inet_ntoa (or->u.std.area_id));
+#endif /* HAVE_NSSA */
 
 	      /* Show flags. */
 	      vty_out (vty, "%s%s%s",
@@ -820,6 +844,8 @@ show_ip_ospf_route_external (struct vty *vty, struct route_table *rt)
 {
   struct route_node *rn;
   struct ospf_route *er;
+  listnode pnode;
+  struct ospf_path *path;
 
   vty_out (vty, "============ OSPF external routing table ===========%s",
 	   VTY_NEWLINE);
@@ -841,10 +867,56 @@ show_ip_ospf_route_external (struct vty *vty, struct route_table *rt)
 		     er->u.ext.type2_cost, er->u.ext.tag, VTY_NEWLINE);
 	    break;
 	  }
-      }
+
+        for (pnode = listhead (er->path); pnode; nextnode (pnode))
+          {
+            path = getdata (pnode);
+            if (path->ifp != NULL)
+              {
+                if (path->nexthop.s_addr == 0)
+	          vty_out (vty, "%24s   directly attached to %s%s",
+		           "", path->ifp->name, VTY_NEWLINE);
+                else 
+	          vty_out (vty, "%24s   via %s, %s%s", "",
+			   inet_ntoa (path->nexthop), path->ifp->name,
+     		           VTY_NEWLINE);
+              }
+           }
+        }
   vty_out (vty, "%s", VTY_NEWLINE);
 }
 
+#ifdef HAVE_NSSA
+DEFUN (show_ip_ospf_border_routers,
+       show_ip_ospf_border_routers_cmd,
+       "show ip ospf border-routers",
+       SHOW_STR
+       IP_STR
+       "show all the ABR's and ASBR's\n"
+       "for this area\n")
+{
+  if (ospf_top == NULL)
+    {
+      vty_out (vty, "OSPF is not enabled%s", VTY_NEWLINE);
+      return CMD_SUCCESS;
+    }
+
+  if (ospf_top->new_table == NULL)
+    {
+      vty_out (vty, "No OSPF routing information exist%s", VTY_NEWLINE);
+      return CMD_SUCCESS;
+    }
+
+  /* Show Network routes.
+  show_ip_ospf_route_network (vty, ospf_top->new_table);   */
+
+  /* Show Router routes. */
+  show_ip_ospf_route_router (vty, ospf_top->new_rtrs);
+
+  return CMD_SUCCESS;
+}
+#endif /* HAVE_NSSA */
+
 DEFUN (show_ip_ospf_route,
        show_ip_ospf_route_cmd,
        "show ip ospf route",
@@ -877,7 +949,6 @@ DEFUN (show_ip_ospf_route,
   return CMD_SUCCESS;
 }
 
-
 /* This is 16.4.1 implementation.
    o Intra-area paths using non-backbone areas are always the most preferred.
    o The other paths, intra-area backbone paths and inter-area paths,
@@ -916,7 +987,8 @@ ospf_route_cmp (struct ospf_route *r1, struct ospf_route *r2)
   if ((ret = (r1->path_type - r2->path_type)))
     return ret;
 
-  zlog_info ("Route[Compare]: Path types are the same.");
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info ("Route[Compare]: Path types are the same.");
   /* Path types are the same, compare any cost. */
   switch (r1->path_type)
     {
@@ -948,6 +1020,23 @@ ospf_route_cmp (struct ospf_route *r1, struct ospf_route *r2)
   return (r1->cost - r2->cost);
 }
 
+int
+ospf_path_exist (struct list *plist, struct in_addr nexthop,
+		 struct interface *ifp)
+{
+  listnode node;
+  struct ospf_path *path;
+
+  for (node = listhead (plist); node; nextnode (node))
+    {
+      path = node->data;
+
+      if (IPV4_ADDR_SAME (&path->nexthop, &nexthop) && path->ifp == ifp)
+	return 1;
+    }
+  return 0;
+}
+
 void
 ospf_route_copy_nexthops_from_vertex (struct ospf_route *to,
 				      struct vertex *v)
@@ -957,7 +1046,7 @@ ospf_route_copy_nexthops_from_vertex (struct ospf_route *to,
   struct vertex_nexthop *nexthop;
 
   if (to->path == NULL)
-    to->path = list_init ();
+    to->path = list_new ();
 
   for (nnode = listhead (v->nexthop); nnode; nextnode (nnode))
     {
@@ -965,20 +1054,23 @@ ospf_route_copy_nexthops_from_vertex (struct ospf_route *to,
 
       if (nexthop->ifp != NULL) 
 	{
-	  path = ospf_path_new ();
-	  path->nexthop = nexthop->router;
-	  path->ifp = nexthop->ifp;
-	  list_add_node (to->path, path);
+	  if (! ospf_path_exist (to->path, nexthop->router, nexthop->ifp))
+	    {
+	      path = ospf_path_new ();
+	      path->nexthop = nexthop->router;
+	      path->ifp = nexthop->ifp;
+	      listnode_add (to->path, path);
+	    }
 	}
     }
 }
 
 struct ospf_path *
-ospf_path_lookup (list list, struct ospf_path *path)
+ospf_path_lookup (list plist, struct ospf_path *path)
 {
   listnode node;
 
-  for (node = listhead (list); node; nextnode (node))
+  for (node = listhead (plist); node; nextnode (node))
     {
       struct ospf_path *op = node->data;
 
@@ -996,12 +1088,12 @@ ospf_route_copy_nexthops (struct ospf_route *to, list from)
   listnode node;
 
   if (to->path == NULL)
-    to->path = list_init ();
+    to->path = list_new ();
 
   for (node = listhead (from); node; nextnode (node))
     /* The same routes are just discarded. */
     if (!ospf_path_lookup (to->path, node->data))
-      list_add_node (to->path, ospf_path_dup (node->data));
+      listnode_add (to->path, ospf_path_dup (node->data));
 }
 
 void
@@ -1045,7 +1137,8 @@ ospf_route_add (struct route_table *rt, struct prefix_ipv4 *p,
 
   if (rn->info)
     {
-      zlog_info ("ospf_route_add(): something's wrong !");
+      if (IS_DEBUG_OSPF_EVENT)
+	zlog_info ("ospf_route_add(): something's wrong !");
       route_unlock_node (rn);
       return;
     }
@@ -1053,14 +1146,14 @@ ospf_route_add (struct route_table *rt, struct prefix_ipv4 *p,
   rn->info = new_or;
 }
 
-
 void
 ospf_prune_unreachable_networks (struct route_table *rt)
 {
   struct route_node *rn, *next;
   struct ospf_route *or;
 
-  zlog_info ("Pruning unreachable networks");
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info ("Pruning unreachable networks");
 
   for (rn = route_top (rt); rn; rn = next)
     {
@@ -1070,8 +1163,9 @@ ospf_prune_unreachable_networks (struct route_table *rt)
 	  or = rn->info;
 	  if (listcount (or->path) == 0)
 	    {
-	      zlog_info ("Pruning route to %s/%d",
-			 inet_ntoa (rn->p.u.prefix4), rn->p.prefixlen);
+	      if (IS_DEBUG_OSPF_EVENT)
+		zlog_info ("Pruning route to %s/%d",
+			   inet_ntoa (rn->p.u.prefix4), rn->p.prefixlen);
 
 	      ospf_route_free (or);
 	      rn->info = NULL;
@@ -1089,7 +1183,8 @@ ospf_prune_unreachable_routers (struct route_table *rtrs)
   listnode node, nnext;
   list paths;
 
-  zlog_info ("Pruning unreachable routers");
+  if (IS_DEBUG_OSPF_EVENT)
+    zlog_info ("Pruning unreachable routers");
 
   for (rn = route_top (rtrs); rn; rn = next)
     {
@@ -1105,21 +1200,25 @@ ospf_prune_unreachable_routers (struct route_table *rtrs)
 
 	  if (listcount (or->path) == 0)
 	    {
-	      zlog_info ("Pruning route to rtr %s",
-			 inet_ntoa (rn->p.u.prefix4));
-	      zlog_info ("               via area %s",
-			 inet_ntoa (or->u.std.area_id));
+	      if (IS_DEBUG_OSPF_EVENT)
+		{
+		  zlog_info ("Pruning route to rtr %s",
+			     inet_ntoa (rn->p.u.prefix4));
+		  zlog_info ("               via area %s",
+			     inet_ntoa (or->u.std.area_id));
+		}
 
-	      list_delete_by_val (paths, or);
+	      listnode_delete (paths, or);
 	      ospf_route_free (or);
 	    }
 	}
 
       if (listcount (paths) == 0)
 	{
-	  zlog_info ("Pruning router node %s", inet_ntoa (rn->p.u.prefix4));
+	  if (IS_DEBUG_OSPF_EVENT)
+	    zlog_info ("Pruning router node %s", inet_ntoa (rn->p.u.prefix4));
 
-	  list_delete_all (paths);
+	  list_delete (paths);
 	  rn->info = NULL;
 	  route_unlock_node (rn);
 	}
@@ -1137,7 +1236,8 @@ ospf_add_discard_route (struct route_table *rt, struct ospf_area *area,
 
   if (rn == NULL)
     {
-      zlog_info ("ospf_add_discard_route(): router installation error");
+      if (IS_DEBUG_OSPF_EVENT)
+	zlog_info ("ospf_add_discard_route(): router installation error");
       return 0;
     }
 
@@ -1149,15 +1249,17 @@ ospf_add_discard_route (struct route_table *rt, struct ospf_area *area,
 
       if (or->path_type == OSPF_PATH_INTRA_AREA)
 	{
-	  zlog_info ("ospf_add_discard_route(): "
-		     "an intra-area route exists");
+	  if (IS_DEBUG_OSPF_EVENT)
+	    zlog_info ("ospf_add_discard_route(): "
+		       "an intra-area route exists");
 	  return 0;
 	}
 
       if (or->type == OSPF_DESTINATION_DISCARD)
 	{
-	  zlog_info ("ospf_add_discard_route(): "
-		     "discard entry already installed");
+	  if (IS_DEBUG_OSPF_EVENT)
+	    zlog_info ("ospf_add_discard_route(): "
+		       "discard entry already installed");
 	  return 0;
 	}
 
@@ -1169,6 +1271,9 @@ ospf_add_discard_route (struct route_table *rt, struct ospf_area *area,
   new_or->id.s_addr = 0;
   new_or->cost = 0;
   new_or->u.std.area_id = area->area_id;
+#ifdef HAVE_NSSA
+  new_or->u.std.external_routing = area->external_routing;
+#endif /* HAVE_NSSA */
   new_or->path_type = OSPF_PATH_INTER_AREA;
   rn->info = new_or;
 
@@ -1187,5 +1292,12 @@ void
 ospf_route_init ()
 {
   install_element (VIEW_NODE, &show_ip_ospf_route_cmd);
+#ifdef HAVE_NSSA
+  install_element (VIEW_NODE, &show_ip_ospf_border_routers_cmd);
+#endif /* HAVE_NSSA */
   install_element (ENABLE_NODE, &show_ip_ospf_route_cmd);
+#ifdef HAVE_NSSA
+  install_element (ENABLE_NODE, &show_ip_ospf_border_routers_cmd);
+#endif /* HAVE_NSSA */
+
 }
