@@ -69,6 +69,11 @@ struct route_standard
   /* Associated Area. */
   struct in_addr area_id;	/* The area the route belongs to */
 
+#ifdef HAVE_NSSA
+  /*  Area Type */
+  int external_routing;
+#endif /* HAVE_NSSA */
+
   /* Optional Capability. */
   u_char options;		/* Get from LSA header. */
 
@@ -251,7 +256,7 @@ void ospf_prune_unreachable_routers (struct route_table *);
 int ospf_add_discard_route (struct route_table *, struct ospf_area *, 
 			    struct prefix_ipv4 *);
 void ospf_delete_discard_route (struct prefix_ipv4 *);
-int ospf_route_match_same (struct route_table *, int,
-			   struct prefix_ipv4 *, struct in_addr *);
+int ospf_route_match_same (struct route_table *, struct prefix_ipv4 *,
+			   struct ospf_route *);
 
 #endif /* _ZEBRA_OSPF_ROUTE_H */
