@@ -44,8 +44,9 @@
 #include "ospfd/ospf_dump.h"
 #include "ospfd/ospf_route.h"
 #include "ospfd/ospf_zebra.h"
-
-extern unsigned long term_debug_ospf_zebra;
+#ifdef HAVE_SNMP
+#include "ospfd/ospf_snmp.h"
+#endif /* HAVE_SNMP */
 
 /* Zebra structure to hold current status. */
 struct zclient *zclient = NULL;
@@ -67,6 +68,10 @@ ospf_interface_add (int command, struct zclient *zclient, zebra_size_t length)
 
   ospf_if_update ();
 
+#ifdef HAVE_SNMP
+  ospf_snmp_if_update (ifp);
+#endif /* HAVE_SNMP */
+
   return 0;
 }
 
@@ -95,18 +100,11 @@ ospf_interface_delete (int command, struct zclient *zclient,
     zlog_info ("Zebra: interface delete %s index %d flags %d metric %d mtu %d",
 	       ifp->name, ifp->ifindex, ifp->flags, ifp->metric, ifp->mtu);  
 
-#ifdef HAVE_IF_PSEUDO
-  if (!IS_IF_PSEUDO(ifp))
-    {
-      if_delete(ifp);
-    }
-  else
-    {
-      ifp->ifindex=INTERFACE_PSEUDO;
-    }
-#else
+#ifdef HAVE_SNMP
+  ospf_snmp_if_delete (ifp);
+#endif /* HAVE_SNMP */
+
   if_delete(ifp);
-#endif /* HAVE_IF_PSEUDO */
 
   return 0;
 }
@@ -141,13 +139,6 @@ zebra_interface_if_set_value (struct stream *s, struct interface *ifp)
   ifp->metric = stream_getl (s);
   ifp->mtu = stream_getl (s);
   ifp->bandwidth = stream_getl (s);
-
-#ifdef IS_IF_PSEUDO
-  if (IS_IF_PSEUDO (ifp))
-    IF_PSEUDO_SET (ifp);
-  else
-    IF_PSEUDO_UNSET (ifp);
-#endif
 }
 
 int
@@ -184,7 +175,7 @@ ospf_interface_state_up (int command, struct zclient *zclient,
 	  oi = ifp->info;
 	  old_cost = oi->output_cost;
 	  oi->output_cost = ospf_if_get_output_cost (oi);
-	  if (old_cost != oi->output_cost)
+	  if (old_cost != oi->output_cost && oi->area != NULL)
 	    ospf_router_lsa_timer_add (oi->area);
 	}
       return 0;
@@ -244,6 +235,10 @@ ospf_interface_address_add (int command, struct zclient *zclient,
 
   ospf_if_update ();
 
+#ifdef HAVE_SNMP
+  ospf_snmp_if_update (c->ifp);
+#endif /* HAVE_SNMP */
+
   return 0;
 }
 
@@ -251,37 +246,113 @@ int
 ospf_interface_address_delete (int command, struct zclient *zclient,
 			       zebra_size_t length)
 {
+  struct connected *c;
+  struct interface *ifp;
+  struct ospf_interface *oi;
+
+  c = zebra_interface_address_delete_read (zclient->ibuf);
+
+  if (c == NULL)
+    return 0;
+
+  ifp = c->ifp;
+  oi = ifp->info;
+
+  /* Clean up if address was our primary */
+  if (oi->address && prefix_match (c->address, oi->address))
+    {
+      /* Call interface hook functions to clean up */
+      ospf_if_cleanup(oi, 0);
+    }
+
+#ifdef HAVE_SNMP
+  ospf_snmp_if_update (c->ifp);
+#endif /* HAVE_SNMP */
+
+  connected_free (c);
+
+  /* Check ffor another subnet that could take over */
+  ospf_if_update();
+
   return 0;
 }
-
 
 void
-ospf_zebra_add (struct prefix_ipv4 *p, struct in_addr *nexthop,
-		u_int32_t metric, struct ospf_route *or)
+ospf_zebra_add (struct prefix_ipv4 *p, struct ospf_route *or)
 {
+  u_char message;
   u_char distance;
-  struct zapi_ipv4 api;
+  u_char flags;
+  int psize;
+  struct stream *s;
+  struct ospf_path *path;
+  listnode node;
 
   if (zclient->redist[ZEBRA_ROUTE_OSPF])
     {
-      api.type = ZEBRA_ROUTE_OSPF;
-      api.flags = 0;
-      api.message = 0;
-      SET_FLAG (api.message, ZAPI_MESSAGE_NEXTHOP);
-      api.nexthop_num = 1;
-      api.nexthop = &nexthop;
-      SET_FLAG (api.message, ZAPI_MESSAGE_METRIC);
-      api.metric = metric;
+      message = 0;
+      flags = 0;
 
+      /* OSPF pass nexthop and metric */
+      SET_FLAG (message, ZAPI_MESSAGE_NEXTHOP);
+      SET_FLAG (message, ZAPI_MESSAGE_METRIC);
+
+      /* Distance value. */
       distance = ospf_distance_apply (p, or);
       if (distance)
+	SET_FLAG (message, ZAPI_MESSAGE_DISTANCE);
+
+      /* Make packet. */
+      s = zclient->obuf;
+      stream_reset (s);
+
+      /* Length place holder. */
+      stream_putw (s, 0);
+
+      /* Put command, type, flags, message. */
+      stream_putc (s, ZEBRA_IPV4_ROUTE_ADD);
+      stream_putc (s, ZEBRA_ROUTE_OSPF);
+      stream_putc (s, flags);
+      stream_putc (s, message);
+  
+      /* Put prefix information. */
+      psize = PSIZE (p->prefixlen);
+      stream_putc (s, p->prefixlen);
+      stream_write (s, (u_char *)&p->prefix, psize);
+
+      /* Nexthop count. */
+      stream_putc (s, or->path->count);
+
+      /* Nexthop, ifindex, distance and metric information. */
+      for (node = listhead (or->path); node; nextnode (node))
 	{
-	  SET_FLAG (api.message, ZAPI_MESSAGE_DISTANCE);
-	  api.distance = distance;
+	  path = getdata (node);
+
+	  if (path->nexthop.s_addr != INADDR_ANY)
+	    {
+	      stream_putc (s, ZEBRA_NEXTHOP_IPV4);
+	      stream_put_in_addr (s, &path->nexthop);
+	    }
+	  else
+	    {
+	      stream_putc (s, ZEBRA_NEXTHOP_IFINDEX);
+	      if (path->ifp)
+		stream_putl (s, path->ifp->ifindex);
+	      else
+		stream_putl (s, 0);
+	    }
 	}
 
-      zapi_ipv4_add (zclient, p, &api);
+      if (CHECK_FLAG (message, ZAPI_MESSAGE_DISTANCE))
+	stream_putc (s, distance);
+      if (CHECK_FLAG (message, ZAPI_MESSAGE_METRIC))
+	stream_putl (s, or->cost);
 
+      stream_putw_at (s, 0, stream_get_endp (s));
+
+      writen (zclient->sock, s->data, stream_get_endp (s));
+
+#if 0
       if (IS_DEBUG_OSPF (zebra, ZEBRA_REDISTRIBUTE))
 	{
 	  char *nexthop_str;
@@ -292,11 +363,12 @@ ospf_zebra_add (struct prefix_ipv4 *p, struct in_addr *nexthop,
 		     metric);
 	  free (nexthop_str);
 	}
+#endif /* 0 */
     }
 }
 
 void
-ospf_zebra_delete (struct prefix_ipv4 *p, struct in_addr *nexthop)
+ospf_zebra_delete (struct prefix_ipv4 *p, struct ospf_route *or)
 {
   struct zapi_ipv4 api;
 
@@ -305,16 +377,19 @@ ospf_zebra_delete (struct prefix_ipv4 *p, struct in_addr *nexthop)
       api.type = ZEBRA_ROUTE_OSPF;
       api.flags = 0;
       api.message = 0;
+      /* Do not send nexthop information when delete route.
       SET_FLAG (api.message, ZAPI_MESSAGE_NEXTHOP);
       api.nexthop_num = 1;
       api.nexthop = &nexthop;
+      api.ifindex_num = 0;
+      */
       /* Do not pass metric when delete route. 
 	SET_FLAG (api.message, ZAPI_MESSAGE_METRIC);
 	api.metric = metric;
       */
-
       zapi_ipv4_delete (zclient, p, &api);
 
+#if 0
       if (IS_DEBUG_OSPF (zebra, ZEBRA_REDISTRIBUTE))
 	{
 	  char *nexthop_str;
@@ -324,6 +399,7 @@ ospf_zebra_delete (struct prefix_ipv4 *p, struct in_addr *nexthop)
 		     inet_ntoa (p->prefix), p->prefixlen, nexthop_str);
 	  free (nexthop_str);
 	}
+#endif /* 0 */
     }
 }
 
@@ -345,6 +421,7 @@ ospf_zebra_add_discard (struct prefix_ipv4 *p)
       SET_FLAG (api.message, ZAPI_MESSAGE_NEXTHOP);
       api.nexthop_num = 1;
       api.nexthop = &nexthop;
+      api.ifindex_num = 0;
 
       zapi_ipv4_add (zclient, p, &api);
     }
@@ -368,6 +445,7 @@ ospf_zebra_delete_discard (struct prefix_ipv4 *p)
       SET_FLAG (api.message, ZAPI_MESSAGE_NEXTHOP);
       api.nexthop_num = 1;
       api.nexthop = &nexthop;
+      api.ifindex_num = 0;
 
       zapi_ipv4_delete (zclient, p, &api);
     }
@@ -692,8 +770,16 @@ ospf_zebra_read_ipv4 (int command, struct zclient *zclient,
 		ospf_external_lsa_refresh_default ();
 	      else
 		{
-		  if (!ospf_external_info_find_lsa (&ei->p))
+		  struct ospf_lsa *current;
+
+		  current = ospf_external_info_find_lsa (&ei->p);
+		  if (!current)
 		    ospf_external_lsa_originate (ei);
+		  else if (IS_LSA_MAXAGE (current))
+		    ospf_external_lsa_refresh (current, ei, LSA_REFRESH_FORCE);
+		  else
+		    zlog_warn ("ospf_zebra_read_ipv4() : %s already exists",
+			       inet_ntoa (p.prefix));
 		}
 	    }
 	}
@@ -735,49 +821,6 @@ DEFUN (no_router_zebra,
   return CMD_SUCCESS;
 }
 
-#if 0
-DEFUN (ospf_redistribute_ospf,
-       ospf_redistribute_ospf_cmd,
-       "redistribute ospf",
-       "Redistribute information from another routing protocol\n"
-       "Open Shortest Path First (OSPF)\n")
-{
-  zclient->redist[ZEBRA_ROUTE_OSPF] = 1;
-  return CMD_SUCCESS;
-}
-
-DEFUN (no_ospf_redistribute_ospf,
-       no_ospf_redistribute_ospf_cmd,
-       "no redistribute ospf",
-       NO_STR
-       "Redistribute information from another routing protocol\n"
-       "Open Shortest Path First (OSPF)\n")
-{
-  zclient->redist[ZEBRA_ROUTE_OSPF] = 0;
-  return CMD_SUCCESS;
-}
-
-DEFUN (ospf_redistribute_kernel,
-       ospf_redistribute_kernel_cmd,
-       "redistribute kernel",
-       "Redistribute information from another routing protocol\n"
-       "Kernel routes\n")
-{
-  return ospf_redistribute_set (ZEBRA_ROUTE_KERNEL, EXTERNAL_METRIC_TYPE_2,
-				OSPF_EXT_METRIC_AUTO, 0);
-}
-
-DEFUN (no_ospf_redistribute_kernel,
-       no_ospf_redistribute_kernel_cmd,
-       "no redistribute kernel",
-       NO_STR
-       "Redistribute information from another routing protocol\n"
-       "Kernel routes\n")
-{
-  return ospf_redistribute_unset (ZEBRA_ROUTE_KERNEL);
-}
-#endif
-
 int
 str2distribute_source (char *str, int *source)
 {
