@@ -36,9 +36,6 @@
 
 #include "zebra/connected.h"
 
-#ifdef RIP_API
-#include "ripd/ripd_api.h"
-#endif /* RIP_API */
 #include "ripd/ripd.h"
 #include "ripd/rip_debug.h"
 
@@ -77,7 +74,8 @@ ipv4_multicast_join (int sock, struct in_addr group, struct in_addr ifa)
 		    (char *)&mreq, sizeof (mreq));
 
   if (ret < 0) 
-    zlog (NULL, LOG_INFO, "can't setsockopt IP_ADD_MEMBERSHIP");
+    zlog (NULL, LOG_INFO, "can't setsockopt IP_ADD_MEMBERSHIP %s",
+	  strerror (errno));
 
   return ret;
 }
@@ -303,7 +301,7 @@ rip_request_neighbor_all ()
 }
 
 /* Multicast packet receive socket. */
-void
+int
 rip_multicast_join (struct interface *ifp, int sock)
 {
   listnode cnode;
@@ -326,9 +324,11 @@ rip_multicast_join (struct interface *ifp, int sock)
 	    continue;
       
 	  group.s_addr = htonl (INADDR_RIP_GROUP);
-	  ipv4_multicast_join (sock, group, p->prefix);
+	  if (ipv4_multicast_join (sock, group, p->prefix) < 0)
+	    return -1;
 	}
     }
+  return 0;
 }
 
 /* Leave from multicast group. */
@@ -360,7 +360,7 @@ rip_multicast_leave (struct interface *ifp, int sock)
     }
 }
 
-/* Does this address belong to me ? */
+/* Does this address belongs to me ? */
 int
 if_check_address (struct in_addr addr)
 {
@@ -383,6 +383,7 @@ if_check_address (struct in_addr addr)
 
 	  if (p->family != AF_INET)
 	    continue;
+
 	  if (IPV4_ADDR_CMP (&p->prefix, &addr) == 0)
 	    return 1;
 	}
@@ -497,6 +498,9 @@ rip_interface_up (int command, struct zclient *zclient, zebra_size_t length)
   /* Check if this interface is RIP enabled or not.*/
   rip_enable_apply (ifp);
  
+  /* Check for a passive interface */
+  rip_passive_interface_apply (ifp);
+
   /* Apply distribute list to the all interface. */
   rip_distribute_update_interface (ifp);
 
@@ -548,16 +552,8 @@ rip_interface_delete (int command, struct zclient *zclient,
   zlog_info("interface delete %s index %d flags %d metric %d mtu %d",
 	    ifp->name, ifp->ifindex, ifp->flags, ifp->metric, ifp->mtu);  
   
-#ifdef HAVE_IF_PSEUDO
-  if (!IS_IF_PSEUDO(ifp)){
-     if_delete(ifp);
-   }
-   else{
-     ifp->ifindex=INTERFACE_PSEUDO;
-   }
-#else
   if_delete(ifp);
-#endif /* HAVE_IF_PSEUDO */
+
   return 0;
 }
 
@@ -647,47 +643,6 @@ rip_if_down(struct interface *ifp)
   struct route_node *rp;
   struct rip_info *rinfo;
   struct rip_interface *ri = NULL;
-#ifdef NEW_RIP_TABLE
-  struct rip_route *route;
-  struct rip_info *next;
-
-  /* Clear RIP dynamic routes on the interface*/
-  if (rip)
-    {
-      for (rp = route_top (rip->table); rp; rp = route_next (rp))
-	if ((route = rp->info) != NULL)
-	  for (rinfo = route->head; rinfo; rinfo = next)
-	    {
-	      next = rinfo->next;
-
-	      /* Routes got through this interface. */
-	      if (rinfo->ifindex == ifp->ifindex &&
-		  rinfo->type == ZEBRA_ROUTE_RIP &&
-		  rinfo->sub_type == RIP_ROUTE_RTE)
-		{
-		  rip_zebra_ipv4_delete ((struct prefix_ipv4 *) &rp->p,
-					 &rinfo->nexthop,
-					 rinfo->ifindex);
-
-		  RIP_TIMER_OFF (rinfo->t_timeout);
-		  RIP_TIMER_OFF (rinfo->t_garbage_collect);
-	      
-		  route_unlock_node (rp);
-		  rip_info_free (rinfo);
-		}
-	      else
-		{
-		  /* All redistributed routes but static and system */
-		  if ((rinfo->ifindex == ifp->ifindex) &&
-		      (rinfo->type != ZEBRA_ROUTE_STATIC) &&
-		      (rinfo->type != ZEBRA_ROUTE_SYSTEM))
-		    rip_redistribute_delete (rinfo->type,rinfo->sub_type,
-					     (struct prefix_ipv4 *)&rp->p,
-					     rinfo->ifindex);
-		}
-	    }
-    }
-#else
   if (rip)
     {
       for (rp = route_top (rip->table); rp; rp = route_next (rp))
@@ -722,7 +677,6 @@ rip_if_down(struct interface *ifp)
 	      }
 	  }
     }
-#endif /* NEW_RIP_TABLE */
 	    
   ri = ifp->info;
   
@@ -824,12 +778,12 @@ rip_interface_address_delete (int command, struct zclient *zclient,
 int
 rip_enable_network_lookup (struct interface *ifp)
 {
-  listnode listnode;
+  struct listnode *nn;
   struct connected *connected;
   struct prefix_ipv4 address;
 
-  for (listnode = listhead (ifp->connected); listnode; nextnode (listnode))
-    if ((connected = getdata (listnode)) != NULL)
+  for (nn = listhead (ifp->connected); nn; nextnode (nn))
+    if ((connected = getdata (nn)) != NULL)
       {
 	struct prefix *p; 
 	struct route_node *node;
@@ -956,7 +910,14 @@ rip_interface_wakeup (struct thread *t)
   ri->t_wakeup = NULL;
 
   /* Join to multicast group. */
-  rip_multicast_join (ifp, rip->sock);
+  if (rip_multicast_join (ifp, rip->sock) < 0)
+    {
+      zlog_err ("multicast join failed, interface %s not running", ifp->name);
+      return 0;
+    }
+
+  /* Set running flag. */
+  ri->running = 1;
 
   /* Send RIP request to the interface. */
   rip_request_interface (ifp);
@@ -964,41 +925,6 @@ rip_interface_wakeup (struct thread *t)
   return 0;
 }
 
-#ifdef NEW_RIP_TABLE
-void
-rip_interface_route_add (struct interface *ifp)
-{
-  listnode listnode;
-  struct connected *connected;
-  struct prefix *p; 
-  struct route_node *node;
-  struct rip_info *rinfo;
-
-  for (listnode = listhead (ifp->connected); listnode; nextnode (listnode))
-    if ((connected = getdata (listnode)) != NULL)
-      {
-	p = connected->address;
-
-	if (p->family == AF_INET)
-	  {
-	    rinfo = rip_info_new ();
-	    rinfo->type = ZEBRA_ROUTE_CONNECT;
-	    rinfo->sub_type = RIP_ROUTE_INTERFACE;
-	    rinfo->ifindex = ifp->ifindex;
-	    rinfo->metric = 1;
-	    rip_route_add (p, rinfo);
-	    rip_route_process ();
-	  }
-      }
-}
-
-void
-rip_interface_route_delete (struct interface *ifp)
-{
-  ;
-}
-#endif /* NEW_RIP_TABLE */
-
 /* Update interface status. */
 void
 rip_enable_apply (struct interface *ifp)
@@ -1043,11 +969,6 @@ rip_enable_apply (struct interface *ifp)
 	  if (! ri->t_wakeup)
 	    ri->t_wakeup = thread_add_timer (master, rip_interface_wakeup,
 					     ifp, 1);
-	  ri->running = 1;
-
-#ifdef NEW_RIP_TABLE
-	  rip_interface_route_add (ifp);
-#endif /* NEW_RIP_TABLE */
 	}
     }
   else
@@ -1061,10 +982,6 @@ rip_enable_apply (struct interface *ifp)
 	  rip_multicast_leave (ifp, rip->sock);
 
 	  ri->running = 0;
-
-#ifdef NEW_RIP_TABLE
-	  rip_interface_route_delete (ifp);
-#endif /* NEW_RIP_TABLE */
 	}
     }
 }
@@ -1261,9 +1178,10 @@ rip_passive_interface_clean ()
 /* RIP enable network or interface configuration. */
 DEFUN (rip_network,
        rip_network_cmd,
-       "network IPV4_ADDR",
-       "Enable RIP\n"
-       "IP prefix or interface name\n")
+       "network (A.B.C.D/M|WORD)",
+       "Enable routing on an IP network\n"
+       "IP prefix <network>/<length>, e.g., 35.0.0.0/8\n"
+       "Interface name\n")
 {
   int ret;
   struct prefix_ipv4 p;
@@ -1290,10 +1208,11 @@ DEFUN (rip_network,
 /* RIP enable network or interface configuration. */
 DEFUN (no_rip_network,
        no_rip_network_cmd,
-       "no network IPV4_ADDR",
+       "no network (A.B.C.D/M|WORD)",
        NO_STR
-       "Disable RIP\n"
-       "IP prefix or interface name\n")
+       "Enable routing on an IP network\n"
+       "IP prefix <network>/<length>, e.g., 35.0.0.0/8\n"
+       "Interface name\n")
 {
   int ret;
   struct prefix_ipv4 p;
@@ -1321,8 +1240,8 @@ DEFUN (no_rip_network,
 DEFUN (rip_neighbor,
        rip_neighbor_cmd,
        "neighbor A.B.C.D",
-       "RIP neighbor router address specification\n"
-       "Address of the neighbor router\n")
+       "Specify a neighbor router\n"
+       "Neighbor address\n")
 {
   int ret;
   struct prefix_ipv4 p;
@@ -1345,8 +1264,8 @@ DEFUN (no_rip_neighbor,
        no_rip_neighbor_cmd,
        "no neighbor A.B.C.D",
        NO_STR
-       "RIP neighbor router address specification\n"
-       "Address of the neighbor router\n")
+       "Specify a neighbor router\n"
+       "Neighbor address\n")
 {
   int ret;
   struct prefix_ipv4 p;
@@ -1367,10 +1286,10 @@ DEFUN (no_rip_neighbor,
 DEFUN (ip_rip_receive_version,
        ip_rip_receive_version_cmd,
        "ip rip receive version (1|2)",
-       "IP Information\n"
-       "RIP configuration\n"
-       "Set interface's receive RIP version control\n"
-       "RIP version\n"
+       IP_STR
+       "Routing Information Protocol\n"
+       "Advertisement reception\n"
+       "Version control\n"
        "RIP version 1\n"
        "RIP version 2\n")
 {
@@ -1397,10 +1316,10 @@ DEFUN (ip_rip_receive_version,
 DEFUN (ip_rip_receive_version_1,
        ip_rip_receive_version_1_cmd,
        "ip rip receive version 1 2",
-       "IP Information\n"
-       "RIP configuration\n"
-       "Set interface's receive RIP version control\n"
-       "RIP version\n"
+       IP_STR
+       "Routing Information Protocol\n"
+       "Advertisement reception\n"
+       "Version control\n"
        "RIP version 1\n"
        "RIP version 2\n")
 {
@@ -1418,10 +1337,10 @@ DEFUN (ip_rip_receive_version_1,
 DEFUN (ip_rip_receive_version_2,
        ip_rip_receive_version_2_cmd,
        "ip rip receive version 2 1",
-       "IP Information\n"
-       "RIP configuration\n"
-       "Set interface's receive RIP version control\n"
-       "RIP version\n"
+       IP_STR
+       "Routing Information Protocol\n"
+       "Advertisement reception\n"
+       "Version control\n"
        "RIP version 2\n"
        "RIP version 1\n")
 {
@@ -1441,9 +1360,9 @@ DEFUN (no_ip_rip_receive_version,
        "no ip rip receive version",
        NO_STR
        IP_STR
-       "RIP configuration\n"
-       "Set interface's receive RIP version control\n"
-       "RIP version\n")
+       "Routing Information Protocol\n"
+       "Advertisement reception\n"
+       "Version control\n")
 {
   struct interface *ifp;
   struct rip_interface *ri;
@@ -1455,13 +1374,24 @@ DEFUN (no_ip_rip_receive_version,
   return CMD_SUCCESS;
 }
 
+ALIAS (no_ip_rip_receive_version,
+       no_ip_rip_receive_version_num_cmd,
+       "no ip rip receive version (1|2)",
+       NO_STR
+       IP_STR
+       "Routing Information Protocol\n"
+       "Advertisement reception\n"
+       "Version control\n"
+       "Version 1\n"
+       "Version 2\n")
+
 DEFUN (ip_rip_send_version,
        ip_rip_send_version_cmd,
        "ip rip send version (1|2)",
-       "IP Information\n"
-       "RIP configuration\n"
-       "Set interface's send RIP version control\n"
-       "RIP version\n"
+       IP_STR
+       "Routing Information Protocol\n"
+       "Advertisement transmission\n"
+       "Version control\n"
        "RIP version 1\n"
        "RIP version 2\n")
 {
@@ -1488,10 +1418,10 @@ DEFUN (ip_rip_send_version,
 DEFUN (ip_rip_send_version_1,
        ip_rip_send_version_1_cmd,
        "ip rip send version 1 2",
-       "IP Information\n"
-       "RIP configuration\n"
-       "Set interface's send RIP version control\n"
-       "RIP version\n"
+       IP_STR
+       "Routing Information Protocol\n"
+       "Advertisement transmission\n"
+       "Version control\n"
        "RIP version 1\n"
        "RIP version 2\n")
 {
@@ -1509,10 +1439,10 @@ DEFUN (ip_rip_send_version_1,
 DEFUN (ip_rip_send_version_2,
        ip_rip_send_version_2_cmd,
        "ip rip send version 2 1",
-       "IP Information\n"
-       "RIP configuration\n"
-       "Set interface's send RIP version control\n"
-       "RIP version\n"
+       IP_STR
+       "Routing Information Protocol\n"
+       "Advertisement transmission\n"
+       "Version control\n"
        "RIP version 2\n"
        "RIP version 1\n")
 {
@@ -1532,9 +1462,9 @@ DEFUN (no_ip_rip_send_version,
        "no ip rip send version",
        NO_STR
        IP_STR
-       "RIP configuration\n"
-       "Set interface's send RIP version control\n"
-       "RIP version\n")
+       "Routing Information Protocol\n"
+       "Advertisement transmission\n"
+       "Version control\n")
 {
   struct interface *ifp;
   struct rip_interface *ri;
@@ -1546,15 +1476,26 @@ DEFUN (no_ip_rip_send_version,
   return CMD_SUCCESS;
 }
 
+ALIAS (no_ip_rip_send_version,
+       no_ip_rip_send_version_num_cmd,
+       "no ip rip send version (1|2)",
+       NO_STR
+       IP_STR
+       "Routing Information Protocol\n"
+       "Advertisement transmission\n"
+       "Version control\n"
+       "Version 1\n"
+       "Version 2\n")
+
 DEFUN (ip_rip_authentication_mode,
        ip_rip_authentication_mode_cmd,
        "ip rip authentication mode (md5|text)",
        IP_STR
-       "RIP configuration\n"
-       "RIP authentication\n"
-       "RIP authentication mode\n"
-       "MD5 authentication\n"
-       "Simple text authentication\n")
+       "Routing Information Protocol\n"
+       "Authentication control\n"
+       "Authentication mode\n"
+       "Keyed message digest\n"
+       "Clear text authentication\n")
 {
   struct interface *ifp;
   struct rip_interface *ri;
@@ -1580,9 +1521,9 @@ DEFUN (no_ip_rip_authentication_mode,
        "no ip rip authentication mode",
        NO_STR
        IP_STR
-       "RIP configuration\n"
-       "RIP authentication\n"
-       "RIP authentication mode\n")
+       "Routing Information Protocol\n"
+       "Authentication control\n"
+       "Authentication mode\n")
 {
   struct interface *ifp;
   struct rip_interface *ri;
@@ -1601,20 +1542,20 @@ ALIAS (no_ip_rip_authentication_mode,
        "no ip rip authentication mode (md5|text)",
        NO_STR
        IP_STR
-       "RIP configuration\n"
-       "RIP authentication\n"
-       "RIP authentication mode\n"
-       "MD5 authentication\n"
-       "Simple text authentication\n")
+       "Routing Information Protocol\n"
+       "Authentication control\n"
+       "Authentication mode\n"
+       "Keyed message digest\n"
+       "Clear text authentication\n")
 
 DEFUN (ip_rip_authentication_string,
        ip_rip_authentication_string_cmd,
-       "ip rip authentication string STRING",
+       "ip rip authentication string LINE",
        IP_STR
-       "RIP configuration\n"
-       "RIP authentication\n"
-       "RIP authentication string setting\n"
-       "RIP authentication string\n")
+       "Routing Information Protocol\n"
+       "Authentication control\n"
+       "Authentication string\n"
+       "Authentication string\n")
 {
   struct interface *ifp;
   struct rip_interface *ri;
@@ -1645,13 +1586,12 @@ DEFUN (ip_rip_authentication_string,
 
 DEFUN (no_ip_rip_authentication_string,
        no_ip_rip_authentication_string_cmd,
-       "no ip rip authentication string STRING",
+       "no ip rip authentication string",
        NO_STR
        IP_STR
-       "RIP configuration\n"
-       "RIP authentication\n"
-       "RIP authentication string setting\n"
-       "RIP authentication string\n")
+       "Routing Information Protocol\n"
+       "Authentication control\n"
+       "Authentication string\n")
 {
   struct interface *ifp;
   struct rip_interface *ri;
@@ -1669,21 +1609,22 @@ DEFUN (no_ip_rip_authentication_string,
 
 ALIAS (no_ip_rip_authentication_string,
        no_ip_rip_authentication_string2_cmd,
-       "no ip rip authentication string",
+       "no ip rip authentication string LINE",
        NO_STR
        IP_STR
-       "RIP configuration\n"
-       "RIP authentication\n"
-       "RIP authentication string setting\n")
+       "Routing Information Protocol\n"
+       "Authentication control\n"
+       "Authentication string\n"
+       "Authentication string\n")
 
 DEFUN (ip_rip_authentication_key_chain,
        ip_rip_authentication_key_chain_cmd,
-       "ip rip authentication key-chain KEY-CHAIN",
+       "ip rip authentication key-chain LINE",
        IP_STR
-       "RIP configuration\n"
-       "RIP authentication\n"
-       "Key chain configuration\n"
-       "Key chain name\n")
+       "Routing Information Protocol\n"
+       "Authentication control\n"
+       "Authentication key-chain\n"
+       "name of key-chain\n")
 {
   struct interface *ifp;
   struct rip_interface *ri;
@@ -1708,13 +1649,12 @@ DEFUN (ip_rip_authentication_key_chain,
 
 DEFUN (no_ip_rip_authentication_key_chain,
        no_ip_rip_authentication_key_chain_cmd,
-       "no ip rip authentication key-chain KEY-CHAIN",
+       "no ip rip authentication key-chain",
        NO_STR
        IP_STR
-       "RIP configuration\n"
-       "RIP authentication\n"
-       "Key chain configuration\n"
-       "Key chain name\n")
+       "Routing Information Protocol\n"
+       "Authentication control\n"
+       "Authentication key-chain\n")
 {
   struct interface *ifp;
   struct rip_interface *ri;
@@ -1732,12 +1672,13 @@ DEFUN (no_ip_rip_authentication_key_chain,
 
 ALIAS (no_ip_rip_authentication_key_chain,
        no_ip_rip_authentication_key_chain2_cmd,
-       "no ip rip authentication key-chain",
+       "no ip rip authentication key-chain LINE",
        NO_STR
        IP_STR
-       "RIP configuration\n"
-       "RIP authentication\n"
-       "Key chain configuration\n")
+       "Routing Information Protocol\n"
+       "Authentication control\n"
+       "Authentication key-chain\n"
+       "name of key-chain\n")
 
 DEFUN (rip_split_horizon,
        rip_split_horizon_cmd,
@@ -1760,7 +1701,7 @@ DEFUN (no_rip_split_horizon,
        "no ip split-horizon",
        NO_STR
        IP_STR
-       "Do not perform split horizon\n")
+       "Perform split horizon\n")
 {
   struct interface *ifp;
   struct rip_interface *ri;
@@ -1811,11 +1752,6 @@ rip_interface_config_write (struct vty *vty)
 	vty_out (vty, " description %s%s", ifp->desc,
 		 VTY_NEWLINE);
 
-#ifdef HAVE_IF_PSEUDO
-      if (IS_IF_PSEUDO (ifp))
-	 vty_out (vty, " pseudo %s", VTY_NEWLINE);
-#endif /* HAVE_IF_PSEUDO */
-      
       /* Split horizon. */
       if (ri->split_horizon != ri->split_horizon_default)
 	{
@@ -1946,10 +1882,6 @@ rip_if_init ()
   install_default (INTERFACE_NODE);
   install_element (INTERFACE_NODE, &interface_desc_cmd);
   install_element (INTERFACE_NODE, &no_interface_desc_cmd);
-#ifdef HAVE_IF_PSEUDO  
-  install_element (INTERFACE_NODE, &interface_pseudo_cmd);
-  install_element (INTERFACE_NODE, &no_interface_pseudo_cmd);
-#endif /* HAVE_IF_PSEUDO */
   install_element (RIP_NODE, &rip_network_cmd);
   install_element (RIP_NODE, &no_rip_network_cmd);
   install_element (RIP_NODE, &rip_neighbor_cmd);
@@ -1962,11 +1894,13 @@ rip_if_init ()
   install_element (INTERFACE_NODE, &ip_rip_send_version_1_cmd);
   install_element (INTERFACE_NODE, &ip_rip_send_version_2_cmd);
   install_element (INTERFACE_NODE, &no_ip_rip_send_version_cmd);
+  install_element (INTERFACE_NODE, &no_ip_rip_send_version_num_cmd);
 
   install_element (INTERFACE_NODE, &ip_rip_receive_version_cmd);
   install_element (INTERFACE_NODE, &ip_rip_receive_version_1_cmd);
   install_element (INTERFACE_NODE, &ip_rip_receive_version_2_cmd);
   install_element (INTERFACE_NODE, &no_ip_rip_receive_version_cmd);
+  install_element (INTERFACE_NODE, &no_ip_rip_receive_version_num_cmd);
 
   install_element (INTERFACE_NODE, &ip_rip_authentication_mode_cmd);
   install_element (INTERFACE_NODE, &no_ip_rip_authentication_mode_cmd);
@@ -1983,437 +1917,3 @@ rip_if_init ()
   install_element (INTERFACE_NODE, &rip_split_horizon_cmd);
   install_element (INTERFACE_NODE, &no_rip_split_horizon_cmd);
 }
-
-#ifdef RIP_API
-/****** API ****/
-
-/*************************
- Function:ripd_api_enable_ip_if
- PURPOSE: Enable RIP network
- PARAMETERS:
-    IN    struct prefix* prefx : IP prefix
-    IN    u_char enable_it : 0 - disable, else - enable
-**************************/
-int
-ripd_api_enable_ip_if (struct prefix* prefx, u_char enable_it)
-{
-  int ret;
-
-  if (enable_it)
-    ret = rip_enable_network_add (prefx);
-  else
-    ret = rip_enable_network_delete (prefx);
-
-  if (ret < 0)
-    {
-      return CMD_WARNING;
-    }
-
-  rip_enable_apply_all ();
-
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_enable_network
- PURPOSE: Enable RIP network
- PARAMETERS:
-    IN    char* if_name : IP prefix or interface name
-    IN    u_char enable_it : 0 - disable, else - enable
-**************************/
-int
-ripd_api_enable_network (char* if_name, u_char enable_it)
-{
-  int ret;
-  prefix_ipv4_t p;
-
-  ret = str2prefix_ipv4 (if_name, &p);
-  if (ret)
-    return ripd_api_enable_ip_if ((struct prefix *) &p, enable_it);
-
-  if (enable_it)
-    {
-      ret = rip_enable_if_add (if_name);
-    }
-  else
-    {
-      ret = rip_enable_if_delete (if_name);
-    }
-
-  if (ret < 0)
-    {
-      return CMD_WARNING;
-    }
-
-  rip_enable_apply_all ();
-
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_enable_neighbor
- PURPOSE: Enable/Disable RIP neighbor router
- PARAMETERS:
-    IN    struct prefix* prefx : IP prefix
-    IN    u_char enable_it : 0 - disable, else - enable
-**************************/
-int
-ripd_api_enable_neighbor (prefix_ipv4_t* prefx, u_char enable_it)
-{
-  if (enable_it)
-    rip_neighbor_add (prefx);
-  else
-    rip_neighbor_delete (prefx);
-
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_get_if_running
- PURPOSE: Return the flag : is RIP running on this interface.
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be checked
-    OUT   int *running - pointer to result
-**************************/
-int
-ripd_api_get_if_running (interface_t *ifp, int *running)
-{
-  rip_interface_t *ri;
-
-  ri = ifp->info;
-  *running = ri->running;
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_set_if_rx_version
- PURPOSE: Set interface's receive RIP version control
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be configured
-    IN    int ri_version - RIP accepet/announce method
-**************************/
-int
-ripd_api_set_if_rx_version (interface_t *ifp, int ri_version)
-{
-  rip_interface_t *ri;
-
-  ri = ifp->info;
-  ri->ri_receive = ri_version;
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_get_if_rx_version
- PURPOSE: Get interface's receive RIP version control
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be configured
-    OUT   int* ri_version - pointer for result
-**************************/
-int
-ripd_api_get_if_rx_version (interface_t *ifp, int *ri_version)
-{
-  rip_interface_t *ri;
-
-  ri = ifp->info;
-
-  if (! ri->running || !rip || !(ri->enable_network || ri->enable_interface))
-    {
-      *ri_version = 0;
-      return CMD_WARNING;
-    }
-
-  if (ri->ri_receive == RI_RIP_UNSPEC)
-    *ri_version=rip->version;
-  else
-    *ri_version=ri->ri_receive;
-    
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_set_if_tx_version
- PURPOSE: Set interface's send RIP version control
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be configured
-    IN    int ri_version - RIP accepet/announce method
-**************************/
-int
-ripd_api_set_if_tx_version (interface_t *ifp, int ri_version)
-{
-  rip_interface_t *ri;
-
-  ri = ifp->info;
-  ri->ri_send = ri_version;
-
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_get_if_tx_version
- PURPOSE: Get interface's send RIP version control
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be configured
-    OUT   int* ri_version - pointer for result
-**************************/
-int
-ripd_api_get_if_tx_version (interface_t *ifp, int *ri_version)
-{
-  rip_interface_t *ri;
-
-  ri = ifp->info;
-
-  if (! ri->running || !rip || !(ri->enable_network || ri->enable_interface)){    
-    *ri_version = 0;
-    return CMD_WARNING;
-  }
-
-  if (ri->ri_send == RI_RIP_UNSPEC)
-    *ri_version=rip->version;
-  else
-    *ri_version=ri->ri_send;
-    
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_set_if_authentication_type
- PURPOSE: Set RIP authentication type
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be configured
-    IN    int  auth_type - RIPv2 authentication type
-                            (from {RIP_NO_AUTH,
-                                   RIP_AUTH_SIMPLE_PASSWORD,
-                                   RIP_AUTH_MD5})
-**************************/
-int
-ripd_api_set_if_authentication_type (interface_t *ifp, int auth_type)
-{
-  rip_interface_t *ri;
-
-  if (RIP_NO_AUTH != auth_type &&
-      RIP_AUTH_SIMPLE_PASSWORD != auth_type &&
-      RIP_AUTH_MD5 != auth_type)
-    {
-      fprintf (stderr, "invalid auth.type %d\n", auth_type);
-      return CMD_ERR_AMBIGUOUS;
-    }
-
-  ri = ifp->info;
-
-  ri->auth_type = auth_type;
-
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_set_if_authentication_string
- PURPOSE: RIP authentication string setting
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be configured
-    IN    char* auth_str - RIP authentication string;
-                           if auth_str==NULL, authentication string is disabled
-**************************/
-int
-ripd_api_set_if_authentication_string (interface_t* ifp, char* auth_str)
-{
-  rip_interface_t *ri;
-
-  ri = ifp->info;
-
-  if (auth_str && *auth_str)
-    {
-      if (strlen (auth_str) > 16)
-        {
-          zlog_err ("api_set:auth_str is too long (%d)", (int) strlen (auth_str));
-          return CMD_WARNING;
-        }
-
-      if (ri->auth_str)
-        free (ri->auth_str);
-
-      ri->auth_str = strdup (auth_str);
-    }
-  else
-    {
-      if (ri->auth_str)
-        free (ri->auth_str);
-
-      ri->auth_type = RIP_NO_AUTH;
-      ri->auth_str = NULL;
-    }
-
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_get_if_authentication_string
- PURPOSE: GET RIP authentication string
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be configured
-    IN    int buffer_size - size of the buffer for the result
-    OUT   char* auth_str - pointer to buffer for result (actual length
-                           may be counted as its 'strlen')
-    OUT   int*  auth_type - RIPv2 authentication type
-                            (from {RIP_NO_AUTH,
-                                   RIP_AUTH_SIMPLE_PASSWORD,
-                                   RIP_AUTH_MD5})
-**************************/
-int
-ripd_api_get_if_authentication_string (interface_t* ifp,
-                                       int buffer_size,
-                                       char* auth_str,
-                                       int* auth_type)
-{
-  rip_interface_t *ri;
-
-  ri = ifp->info;
-
-  if (! ri)
-    return CMD_WARNING;
-
-  if (auth_type)
-    *auth_type = ri->auth_type;
-
-  if (! auth_str)
-    return CMD_SUCCESS;
-
-  if (ri->auth_str)
-    {
-      strncpy (auth_str, ri->auth_str, buffer_size);
-      if (strlen (ri->auth_str) >= buffer_size)
-        { /* it is too long => cut it ! */
-          zlog_err ("api_get:auth_str is too long, cut it");
-          auth_str[buffer_size - 1] = '\0';
-          return CMD_WARNING; /* the buffer has been cut */
-        }
-    }
-  else
-    {/* build the empty string */
-      //zlog_warn ("api_get:auth_str is empty");
-      auth_str[0] = '\0';
-      if (RIP_NO_AUTH != ri->auth_type)
-        return CMD_WARNING; /* unconsistent */
-    }
-
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_set_if_split_horizon
- PURPOSE: Set interface's send RIP version control
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be configured
-    IN    int new_split_mode_value
-**************************/
-int
-ripd_api_set_if_split_horizon (interface_t *ifp, int new_split_mode_value)
-{
-  rip_interface_t *ri;
-
-  ri = ifp->info;
-  ri->split_horizon = new_split_mode_value;
-
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_get_if_split_horizon
- PURPOSE: Set interface's send RIP version control
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be configured
-    OUT   int* current_split_mode_value
-**************************/
-int
-ripd_api_get_if_split_horizon (interface_t *ifp, int* current_split_mode_value)
-{
-  rip_interface_t *ri;
-
-  ri = ifp->info;
-  *current_split_mode_value = ri->split_horizon;
-
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_set_if_description
- PURPOSE: Set/Delete interface description
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be configured
-    IN    char *description (if NULL => delete)
-**************************/
-int
-ripd_api_set_if_description (interface_t *ifp, char *description)
-{
-  if (ifp->desc) /* anyway, delete old one */
-    XFREE (0, ifp->desc);
-
-  if (description && *description)
-    { /* set non-empty description */
-      ifp->desc = XMALLOC (MTYPE_BUFFER,  1 + strlen (description));
-      strcpy (ifp->desc, description);
-    }
-  else
-    { /* delete description */
-      ifp->desc = NULL;
-    }
-
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ripd_api_get_if_description
- PURPOSE: Get interface description
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be configured
-    IN    int buffer_size
-    OUT   char *description
-**************************/
-int
-ripd_api_get_if_description (interface_t *ifp, int buffer_size,
-			     char *description)
-{
-  if (! description || buffer_size < 2)
-    return CMD_WARNING; /* unconsistent */
-
-  if (ifp->desc)
-    {
-      strncpy (description, ifp->desc, buffer_size);
-      description[buffer_size - 1] = '\0';
-    }
-
-  return CMD_SUCCESS;
-}
-
-/*************************
- Function:ipd_api_get_if_statistics
- PURPOSE: Get RIPD interface statistics
- PARAMETERS:
-    IN    interface_t* ifp - inteface to be configured
-    OUT   u_long *recv_badpackets
-    OUT   u_long *recv_badroutes
-    OUT   u_long *sent_updates
-**************************/
-int
-ripd_api_get_if_statistics (interface_t *ifp,
-                            u_long *recv_badpackets,
-                            u_long *recv_badroutes,
-                            u_long *sent_updates)
-{
-  rip_interface_t *ri;
-
-  ri = ifp->info;
-  if (! ri)
-    return CMD_WARNING; /* unconsistent */
-
-  if (recv_badpackets)
-    *recv_badpackets = ri->recv_badpackets;
-
-  if (recv_badroutes)
-    *recv_badroutes = ri->recv_badroutes;
-
-  if (sent_updates)
-    *sent_updates = ri->sent_updates;
-
-  return CMD_SUCCESS;
-}
-#endif /* RIP_API */
